#ifdef WIN32
  #include <string.h>
  #include <time.h>
  #include <ctype.h>
  #include <windows.h>
#else
  #include <sys/time.h>
  #include <ctype.h>
  #include <time.h>
  #include <stdio.h>
  #include <stdlib.h>
  #include <string.h>
  #include <unistd.h>
  #include <fcntl.h>
  #include <netdb.h>
  #include <stdlib.h>

  #include <netdb.h> 
  #include <arpa/inet.h>
  #include <netinet/in.h>
  #include <sys/types.h>
  #include <sys/socket.h>
  #include <string.h>
#endif

#include <stdarg.h>
#include <ctype.h>
#include "sendcdkeycheck.h"
#include "commandlist.h"
#include "command.h"
#include "display.h"
#include "mylib.h"
#include "input.h"
#include "vars.h"
#include "bot.h"
#include "cas.h"

#include "config.h"

extern Display *getDisp();
extern CommandList commandlist;
extern Input *input;
extern Vars vars;
extern Cas cas;

extern int cmd_Handler( char *, char *);

int hex( char ch) {
  switch( ch) {
    case '1':  return( 1); 
    case '2':  return( 2); 
    case '3':  return( 3); 
    case '4':  return( 4); 
    case '5':  return( 5); 
    case '6':  return( 6);
    case '7':  return( 7);
    case '8':  return( 8);
    case '9':  return( 9);
    case 'A':  return( 10);
    case 'B':  return( 11);
    case 'C':  return( 12);
    case 'D':  return( 13);   
    case 'E':  return( 14);
    case 'F':  return( 15);
    case 'a':  return( 10);
    case 'b':  return( 11);
    case 'c':  return( 12);
    case 'd':  return( 13);
    case 'e':  return( 14);
    case 'f':  return( 15);
    default: return( 0); 
  }
}

unsigned char zenc_crc( unsigned char *buf) {
  unsigned char ch = 0xAA; int n = 1;

  while (*buf)
    ch ^= *buf++ * 69069 ^ n++;

  return( ch < 32 ? ~ch : ch);
}

char *hex_decrypt( char *msg) {
  static char buf[ 1024];

  char *dest = &buf[0];
  char *src = msg + 1;

  // First check if it's hex.
//    if (!(strlen( msg) % 1)) {
//      strcpy( buf, msg);
//      return( buf);
//    }

    while (*src) {
      if (!isxdigit( *src)) {
        strcpy( buf, msg);
        return( buf);
      }
      src++;
    }

  // Now let's decode this.
    src = msg + 1;

    while (*src && *(src + 1)) {
      *dest = hex( *src) * 16 + hex(*(src+1));
      dest++;
      src += 2;
    }

  *dest = 0;

  return( buf);
}

// 0 - seed0
// 1 - seed1
// 2 - crc0 - seed0,seed1,' ',' ',string
// 3 - crc1 - seed0,seed1,crc0,' ',crypted msg
unsigned char *zenc_decrypt( unsigned char *msg) {
  int len = strlen( (char *)msg);
  if (len++ < 4) return( NULL);

  static unsigned char buf[ 1024], buf2[ 1024];

  strcpy( (char *)buf2, (char *)msg);
  bzero( buf, sizeof( buf));

  unsigned char crc0 = msg[2], crc1 = msg[3];

  buf2[3] = ' ';

  if (zenc_crc(buf2) != crc1) return( NULL);
  unsigned int seed = (msg[1] << 8) + msg[0];

  int i = 4;
  unsigned char *tmp = buf2 + 4;
   
  while (*tmp) {
    for (int o = 32; o < 256; o++)
        if ((unsigned char)(((o + seed) % 224) + 32) == *tmp) {
	buf[i] = o;
        break;   
      }
      
    seed = 69069 * seed + buf[i];

    tmp++;
    i++;
  }
  buf[i] = 0;

  buf[0] = buf2[0];
  buf[1] = buf2[1];
  buf[2] = ' ';
  buf[3] = ' ';

  if (zenc_crc( buf) == crc0) {
    memmove( buf, buf + 4, len);
    return( buf);
  }
  return( NULL);
}

char *Decrypt( char *src) {
  char source[ 1024];
  bzero( source, sizeof( source));
  strcpy( source, src);

  static char dest[ 1024];
  int len = source[0] - 64, pos = 0;
  for(int i = 1; i < (len * 2); i += 2)
    dest[pos++] = (source[i + 1] - 63) ^ (source[i] - 100);
  dest[pos] = 0;
  return( dest);
}

char *getStr( char *buf, int start, int num) {
  int size = *(u16 *)(buf + 2);
  char *tmp = buf + start;

  while (--num && (tmp - buf) < size)
    tmp += strlen(tmp) + 1;

  if ((tmp - buf) <= size)
    return( tmp);
  else
    return( 0);
}

char *trim2( char *buf) {
  char *p;
 
  for(p = buf; *p; p++)
    if ((*p == '\r') || (*p == '\n'))
      *p = ' ';

  p = buf;
  while ( isspace( *p))
    p++;
 
  strcpy( buf, p);
 
  int n = strlen( buf);
 
  if (n) {
    p = buf + n - 1;
 
    while ( isspace( *p))
      *p-- = '\0';
  }

  return( buf);
}



Bot::Bot( void) {
  clist = new CList( this);
  #ifdef DISP_GTK
    disp = new Window( this);
  #else
    #ifdef WIN32
      disp = new Window( this);
    #else
      disp = getDisp();
    #endif
  #endif

  lag_spoof = 0;
  lastlag = 100;
  queue = new Enqueue;

  queue->addHandler( this);

  // window = 0;
  bzero( username,      sizeof( username));
  bzero( password,      sizeof( password));
  bzero( server,        sizeof( server));
  bzero( cdkey,         sizeof( cdkey));
  bzero( cdkey2,        sizeof( cdkey2));
  bzero( nick,          sizeof( nick));
  bzero( registered,    sizeof( registered));
  bzero( commandset,    sizeof( commandset));
  bzero( curchannel,    sizeof( curchannel));
  bzero( bindaddy,      sizeof( bindaddy));
  
  bzero( socks4_server, sizeof( socks4_server));
  socks4_port = 1080;
  bzero( socks4_userid, sizeof( socks4_userid));

  spawn = 0;
  display = 1; // 0 = off, 1 = on, 2 = gui

  bzero( homechannel, sizeof( homechannel));

  lastconnect_t = 0;

  session_num = session_key = 0;
  filter = 0;

  lastadd_time = lastidle_time = 0;
  lastadd_num = 0;

  profile_lastval = 0;
  profile_lasttime = 0;

  msg_lasttime = 0;
  latency = 0;

  clientType = 0;

  // Command Variables
    cmdIdle_d = 0;
    cmdIdle_t = 0;
    bzero( cmdIdle_msg, sizeof( cmdIdle_msg));

    cmdMode = 0;

  // Quotes
    firstQuote = 0;
    numQuotes = quoteStatus = 0;
    lastQuote = 0;

  proxy = new Proxy( this);
}

Bot::~Bot( void) {
  if (clist)
    delete clist;
  if (disp && (disp != getDisp()))
    delete disp;

  if (queue)
    delete queue;
}

void Bot::clear( void) {
  
}

int Bot::pkt_connected( void) {
  #ifdef DEBUGBOT
    disp->clrline();
    *disp << "DEBUGBOT: Bot::pkt_connected was called" << endl;
    input->update();
  #endif

  if (Network::pkt_connected()) {
    #ifdef DEBUGBOT
      disp->clrline();
      *disp << "DEBUGBOT: Network::pkt_connected told us we are not really connected" << endl;
      input->update();
    #endif
    return( -1);
  }
  
  disp->clrline();
  *disp << CBWHITE << "Info:  " << CBCYAN << getName() << CWHITE << " has connected." << endl;
  #ifdef DEBUGBOT
    *disp <<          "       " << CBCYAN << getName() << CWHITE << " Sending 0x01 to let them know we are here." << endl;
  #endif
  input->update();

  char ch = 1;
  rawsend( 1, &ch);

  #ifdef DEBUGBOT
    disp->clrline();
    switch (getClientType()) {
      case 3:
        *disp << "       " << CBCYAN << getName() << CWHITE << " Sending [0x50ff] Client Computer Information" << endl;
        break;
      default:
        *disp << "       " << CBCYAN << getName() << CWHITE << " Sending [0x1eff] Client Computer Information" << endl;
    }
    input->update();
  #endif

  switch (getClientType()) {
    case 0: // Warcraft
      *this << (u32)0x00000100 // unk - ver ?
            << (u32)0x00000100 // unk - ver ?
            << (u32)0xc8032dc1 // server ip ? 193.45.3.200 for europe1 ?! why why why
            << (u32)0x04dff085 // client_id
            << (u32)0xa2ebcc1f // client_token
            << ""              // host
            << "";             // user
      sendpkt( 0x1eff);
      break;
    case 1: // Starcraft
      //*this << (u32)0x00000001
      //      << (u32)0x00000001
      //      << (u32)0x83caf03f
      //      << (u32)0x0516ce3a
      //      << (u32)0x76ceabd4
      //      << "ghod"              // host
      //      << "sam";             // user
      //sendpkt( 0x1eff);
      //break;
      *this << (u32)0          // spawn ?
            << (u32)0x49583836 // 68XI
            << (u32)0x53544152 // RATS
            << (u32)0xc5       // client_token
            << (u32)0          // ?
            << (u32)0xf300000a // ?
            << (u32)0x000001e0 // bias
            << (u32)0x00000409 // thingy
            << (u32)0x00000409 // thingy
            << "USA"           //
            << "United States; //
      sendpkt( 0x50ff);
      break;
    case 2: // SCJAP
      *this << (u32)0x00000001
            << (u32)0x00000000
            << (u32)0x00000000
            << (u32)0x00000000
            << (u32)0x00000000
            << ""              // host
            << "";             // user
      sendpkt( 0x1eff);
      break;
/*
//    case 3: // D2Exp
//      *this << (u32)0		 // ? spawn ?
//            << (u32)0x49583836   // 68XI
//            << (u32)0x44325850   // PX2D
//            << (u32)0x09	 // version
//            << (u32)0x00	 // ?
//            << (u32)0x00	 // ?
//            << (u32)0x01e0	 // bias
//            << (u32)0x0409	 // english
//            << (u32)0x0409	 // english
//            << "USA"             //
//      sendpkt( 0x50ff);
//      break;
*/
  }

  #ifdef DEBUGBOT
    disp->clrline();
    *disp << "DEBUGBOT: Bot::pkt_connected is exiting" << endl;
    input->update();
  #endif

  return( 0);
}

void Bot::idle( void) {
  if (getfd() == -1) {

    if (!cas.isReady())
      return;

    //===
    // Per popular request
    if ((time(0) - lastconnect_t) < 3)
      return;

    // TODO: These should be moved to a cleanup routine.
      session_num = session_key = 0;
      filter = 0;
      lastadd_time = lastidle_time = 0;
      lastadd_num = 0;
      profile_lastval = 0;
      profile_lasttime = 0;
      msg_lasttime = 0;
      latency = 0;
      lastping = time( 0);
      
    disp->clrline();
    *disp << CBWHITE << "Info:  " << CBCYAN << getName() << CWHITE << " is not connected. Connecting to " << getServer() << endl;
    input->update();
    #ifdef DEBUGBOT
      disp->clrline();
      *disp << CBWHITE << "DEBUGBOT  " << CWHITE << "The clear/update thingy worked." << endl;
      input->update();
    #endif

    lastconnect_t = time( 0);

    #ifdef DEBUGBOT
      disp->clrline();
      *disp << CBWHITE << "DEBUGBOT  " << CWHITE << "Connecting to server..." << endl;
      input->update();
    #endif

    if (netopen( getServer(), 6112, socks4_server, socks4_port, socks4_userid, bindaddy)) {
      disp->clrline();
      *disp << CBWHITE << "Error: " << CBCYAN << getName() << CBRED <<" was unable to connect to server." << endl;
      input->update();
      return;
    }

    #ifdef DEBUGBOT
      disp->clrline();
      *disp << CBWHITE << "DEBUGBOT  " << CWHITE << "Netopened returned." << endl;
      input->update();
    #endif

    return;
  }
  // return;

  if (!filter) {
    #ifndef DEBUGBOT
      if (lastping &&  ((time(0) - lastping) > 120)) {
        disp->clrline();
        *disp << CBYELLOW << "Sys:   " << CWHITE << "Have not heard from bnet in 120 seconds. Reconnecting." << endl;
        input->update();
        netclose();
      }
    #endif

    //#ifdef DEBUGBOT
    //  disp->clrline();
    //  *disp << CBWHITE << "DEBUGBOT: " << CWHITE << "Bot::Idle() is returning [NO FILTER]" << endl;
    //  input->update();
    //#endif
    return;
  }
  
  checkAdds();

  if (atoi(vars.getVar("auto_rejoin")))
    checkHomeChannel();

  checkOps();
  checkQuotes();
  checkIdle();

  queue->idle();

  //#ifdef DEBUGBOT
  //  disp->clrline();
  //  *disp << CBWHITE << "DEBUGBOT: " << CWHITE << "Bot::Idle() is returning [FILTER]" << endl;
  //  input->update();
  //#endif
}

int Bot::checkAdds( void) {
  if ((time(0) - lastadd_time) > 15) {
    lastadd_time = time(0);
    u32 buf[5] = { 0x001415FF, 0x49583836, 0x5732424E, 0, 0 };
    buf[4] = time( 0);

    switch (getClientType()) {
      case 3: // PX2D
        buf[3] = 0x44325850;
        break;
      case 2: // RTSJ
        buf[3] = 0x4A535452;
        break;
      case 1: // sc
        buf[3] = 0x53544152;
        break;
      default: // wc2
        buf[3] = lastadd_num;
    }

    if (char *tmp = vars.getVar("debug_idle"))
      if (atoi( tmp)) {
        disp->clrline();
        *disp << CBCYAN << "Info:  " << CBWHITE << "Sending a add request" << CWHITE << endl;
        input->update();
      }

    rawsend( 20, (char *)buf);
  }

  if ((time(0) - lastidle_time) > 90) {
    lastidle_time = time(0);
    u32 buf = 0x000400ff;

    if (char *tmp = vars.getVar("debug_idle"))
      if (atoi( tmp)) {
        disp->clrline();
        *disp << CBCYAN << "Info:  " << CBWHITE << "Sending a NULL packet" << CWHITE << endl;
        input->update();
      }

    rawsend( 4, (char *)&buf);
  }
  return( 0);
}

int Bot::checkOps( void) {
  if (!strncasecmp( curchannel, "clan ", 5))
    return( 0);

  if (clist->getSize() != 1)
    return( 0);

  CUser *user = clist->getFirst();
  if (!user) return( 0);
  if (user->getFlags() & 2) return( 0);
    
  if ((time(0) - msg_lasttime) < 6) return( 0);

  *this << "/join The Void";
  sendpkt( 0x0eff);

  char buf[ 1024];
  sprintf( buf, "/join %s", curchannel);
  *this << buf;
  sendpkt( 0x0eff);

  // msg_lasttime = time(0) + 3;

  return( 0);
}

int Bot::checkHomeChannel( void) {
  if (*getHomeChannel())
    if (strcasecmp( getHomeChannel(), getCurChannel()))
      enqueue( 7, "/join %s", getHomeChannel());
  return( 0);
}

int Bot::checkIdle( void) {
  #ifdef DEBUGBRITNEY
    if (!cmdIdle_d) {
      cmdIdle_t = time( 0);
      cmdIdle_d = 60;
    }
  #endif

  if (!cmdIdle_d)
    return( 0);

  if ((time(0) - cmdIdle_t) >= cmdIdle_d) {
    cmdIdle_t = time( 0);

    enqueue( 7, "%s", cmdIdle_msg);
  }
  return( 0);
}

int Bot::pkt_send( int pkt_len, char *pkt_buf) {
  if (!filter) return( -1);
  if (getfd() == -1) return( -1);
  if ((time(0) - msg_lasttime) < 3) return( -1);

  char *txt = (char *)zenc_decrypt( (unsigned char *)pkt_buf);
  disp->clrline();
  if (txt) {
    { char *tmp;
    if ((tmp = vars.getVar("show_bots")) && atoi(tmp) && getName() && *getName())
      *disp << CWHITE << getName() << " ";
    }
    *disp << CWHITE  << "["  << CBBLUE  << printTime()        << CWHITE << "] "
          << CBRED << "zENC" << CBWHITE << "> " << CWHITE << txt << endl;

  } else
    if (strncasecmp( pkt_buf, "/me ", 4))
      *disp << CWHITE  << "["  << CBBLUE << printTime()        << CWHITE << "] "
            << CBWHITE << "> " << CWHITE << pkt_buf << endl;
  input->update();

  if (!strncasecmp( pkt_buf, "/join ", 6)) {
    char buf[ 1024];
    strcpy( buf, pkt_buf);

    char *tmp = gettermn( pkt_buf, 2);
    if (!tmp) return( 0);

    // if (strcasecmp( buf + (tmp - pkt_buf), "The Void"))
    //msg_lasttime = time(0) - 3;

    if (strlen( tmp) > 31)
      tmp[31] = 0;
    *this << (u32)2
          << (buf + (tmp - pkt_buf));
    sendpkt( 0x0cff);
    msg_lasttime = time(0) + atoi( vars.getVar( "rejoin_pause"));
  } else {
    *this << pkt_buf;
    sendpkt( 0x0eff);
    if (atoi(vars.getVar("use_delay")))
      msg_lasttime = time(0);
  }

  return( 0);
}


int Bot::pkt_00( int pkt_len, char *pkt_buf) {
  if (char *tmp = vars.getVar("debug_idle"))
    if (atoi( tmp)) {
      disp->clrline();
      *disp << CBCYAN << "Info:  " << CBWHITE << "Received a NULL packet." << CWHITE << endl;
      input->update();
    }

  return( 0);
}


int Bot::pkt_04( int pkt_len, char *pkt_buf) {
  #ifdef DEBUGBOT
    disp->clrline();
    *disp << "Info:  Received a list of the servers" << endl;
    input->update();
  #endif
  return( 0);
}


int Bot::pkt_05( int pkt_len, char *pkt_buf) {
  // 0000   FF 05 14 00 00 00 00 00 - 00 00 00 00 00 00 00 00   ................
  // 0010   00 00 00 00             -                           ....

  #ifdef DEBUG
    disp->clrline();
    *disp << CBWHITE << "Info:  " << CBCYAN << getName() << CWHITE << "[0x05ff] Version Info Received" << endl;
    input->update();
  #endif

  if (pkt_len != 0x14) {
    disp->clrline();
    *disp << "Error: Packet 0x05ff was " << pkt_len << " bytes long." << endl
          << "       Perhaps bnet has changed and we are sending an old 0x1eff." << endl;
    disp->hexdump( pkt_len, pkt_buf);
    input->update();
    netclose();
    return( -1);
  }
  if (*(u32 *)(pkt_buf + 4) > 2) {
    disp->clrline();
    *disp << "Error: Packet 0x05ff returned " << *(u32 *)(pkt_buf + 4) << " for the server version.!" << endl
         << "       Did battle.net change something MAJOR!? Is our 0x1eff correct?" << endl;
    disp->hexdump( pkt_len, pkt_buf);
    input->update();
    netclose();
    return( -1);
  }


  return( 0);
}


int Bot::pkt_06( int pkt_len, char *pkt_buf) {
  #ifdef DEBUG
    disp->clrline();
    disp << "Info:  [0x06ff] Client auth received." << endl
         << "       Sending [0x07ff] CLIENT_AUTHREQ" << endl;
    input->update();
  #endif

  switch( getClientType()) {
    case 0:
      cas.SendCheckRevWar2( this, pkt_buf + 0x19, pkt_buf + 0x0C); // 0x3DDF1A22);
      break;
    case 1:
      cas.SendCheckRevStar( this, pkt_buf + 0x19, pkt_buf + 0x0C); // 0x3DDF1A22);
      break;
    case 2:
      cas.SendCheckRevSCJap( this, pkt_buf + 0x19, pkt_buf + 0x0C); // 0x3DDF1A22);
      break;
    case 3:
      disp->clrline();
      *disp << "Warning: Received a 0x06ff packet with D2XP !?!?" << endl;
      input->update();
      break;
  }

  return( 0);
}


int Bot::pkt_07( int pkt_len, char *pkt_buf) {

  if (pkt_len < 0x0A) {
    disp->clrline();
    *disp << "Error: Packet 0x07ff was " << pkt_len << " bytes long." << endl
         << "       Something is suspicious jim." << endl;
    input->update();
    netclose();
    return( -1);
  }


  if (*(u32 *)(pkt_buf + 4) != 0x02) {
    disp->clrline();
    *disp << "Error: Packet 0x07ff did not return ok. Return: " << *(u32 *)(pkt_buf + 4) << endl;
    if (*(u32 *)(pkt_buf + 4) == 0x01) {
      *disp << "       From the looks of the packet I would say we need to upgrade." << endl
            << "       There is a file \"" << (pkt_buf + 8) << "\" encoded in the message." << endl;
    }
    *disp << "       Something is suspicious jim." << endl;
    input->update();
    netclose();
    return( -1);
  }


  #ifdef DEBUG
    disp->clrline();
    *disp << "Info:  [0x07ff] Server auth_ok received." << endl
          << "       Sending [0x14ff] I got your UDP packet" << endl
          << "       Sending [0x2dff] Icon Request (Acting like the real deal :D)" << endl;
    input->update();
  #endif


  //switch( getClientType()) {
  //  case 0:
      {
        char *tmp2 = "tenb";
        *this << *(u32 *)tmp2;
        sendpkt( 0x14ff);
      }
      sendpkt( 0x2dff);
  //    break;
  //  case 1:
  //    {
  //      char *tmp2 = "tenb";
  //      *this << *(u32 *)tmp2;
  //      sendpkt( 0x14ff);
  //    }
  //    sendpkt( 0x2dff);
  //    break;      
  //}


  #ifdef DEBUG            
    disp->clrline();
    *disp << "       Sending [0x36ff] CD-Key." << endl;
    input->update();
  #endif

  switch( getClientType()) {
    case 0:
      cas.SendCDKeyWar2( this, getCDKey(), session_key, getRegister(), rand(), getSpawn()); // 0x3DDF1A22);
      break;
    case 1:
      cas.SendCDKeyStar( this, getCDKey(), session_key, getRegister(), rand(), getSpawn()); // 0x3DDF1A22);
      break;
    case 2:
      cas.SendCDKeyStar( this, getCDKey(), session_key, getRegister(), rand(), getSpawn()); // 0x3DDF1A22);
      break;
  }

  return( 0);
}


int Bot::pkt_13( int pkt_len, char *pkt_buf) {
  if (pkt_len != 4) {
    disp->clrline();
    *disp << CBYELLOW << "Received a 0x13ff packet with a funky length." << CWHITE << endl;
    disp->hexdump( pkt_len, pkt_buf);
    input->update();
  } else {
    disp->clrline();
    *disp << CBYELLOW << "Battle.net told us to get lost. (Probably for flooding)" << CWHITE << endl;
    input->update();
  }
  netclose();
  return( 0);
}

int Bot::pkt_0a( int pkt_len, char *pkt_buf) {
  // TODO !!!

  // disp.clrline();
  // disp << "Info:  Received a 0A packet >;-)" << endl;
  // disp.hexdump( pkt_len, pkt_buf);
  // input.update();

  return( 0);
}


int Bot::pkt_0b( int pkt_len, char *pkt_buf) {
  #ifdef DEBUGBOT
    disp->clrline();
    *disp << "Info:  Received a list of the public channels" << endl;
    input->update();
  #endif
  return( 0);
}


int Bot::pkt_0c( int pkt_len, char *pkt_buf) {
  // 0000   FF 0C 0E 00 02 00 00 00 - 62 6C 6F 6F 64 00         ........blood.
  // 0000   FF 0C 10 00 02 00 00 00 - 24 42 4C 4F 4F 44 24 00   ........$BLOOD$.
  #ifdef DEBUGBOT
    disp->clrline();
    *disp << "Info:  Received a raw join packet" << endl;
    input->update();
  #endif
  return( 0);
}


int Bot::pkt_0f_01( int pkt_len, char *pkt_buf) {
  // Users in a channel (on channel join)
  //   0000   FF 0F 40 00 01 00 00 00 - 00 00 00 00 6D 00 00 00   ..@.........m...
  //   0010   00 00 00 00 0D F0 AD BA - 0D F0 AD BA 4B 69 6C 6C   ............Kill
  //   0020   65 72 5F 4B 61 68 6E 00 - 4E 42 32 57 20 30 20 30   er_Kahn.NB2W 0 0
  //   0030   20 32 31 20 30 20 30 20 - 30 20 30 20 30 20 30 00    21 0 0 0 0 0 0.

    filter = 1;

    CUser *cuser = new CUser;
    cuser->setName( pkt_buf + 0x1C);
    cuser->setFlags( *(u32 *)(pkt_buf + 0x08));
    cuser->setLatency( *(s32 *)(pkt_buf + 0x0C));
    cuser->setProduct( *(u32 *)(pkt_buf + 0x1c + strlen( pkt_buf + 0x1c) + 1));

    clist->addUser( cuser);
    disp->addUser( cuser->getName(), cuser->getLatency());

    switch( display) {
      case 0: break;
      case 1:
        disp->clrline();

        char *tmp;
        if ((tmp = vars.getVar("show_bots")) && atoi(tmp) && getName() && *getName())
          *disp << CWHITE << getName() << " ";

        *disp << CWHITE  << "["    << CBBLUE   << printTime()        << CWHITE << "] "
             << CBWHITE << "*** " << CBPURPLE << cuser->getName()    << " "
             << CBWHITE << " ["   << CBCYAN   << cuser->getLatency() << CBGREEN << "ms" << CBWHITE << "] "
             << CBWHITE << " ["   << CBGREEN  << cuser->getProduct() << CBWHITE << "] "
             << CBWHITE << " ["   << CBGREEN  << cuser->getFlags() << CBWHITE << "] " << CWHITE << endl;

        input->update();
        break;
    }

  return( 0);
}


// User has joined the channel.
int Bot::pkt_0f_02( int pkt_len, char *pkt_buf) {
  #ifdef DEBUGBOT
    disp->clrline();
    *disp << "DEBUGBOT: Join packet called (IGNOREING IGNORING)" << endl;
    input->update();
    return( 0);
  #endif

  // Add the User

    CUser *cuser = new CUser;
    cuser->setName( pkt_buf + 0x1C);
    cuser->setFlags( *(u32 *)(pkt_buf + 0x08));
    cuser->setLatency( *(s32 *)(pkt_buf + 0x0C));
    cuser->setProduct( *(u32 *)(pkt_buf + 0x1c + strlen( pkt_buf + 0x1c) + 1));

    clist->addUser( cuser);
    disp->addUser( cuser->getName(), cuser->getLatency());


  // Handle the Display
    if (char *tmp = vars.getVar("show_joins"))
      if (atoi(tmp)) {
        if (tmp = vars.getVar("beep_onjoin"))
          if (atoi(tmp))
            printf("\a");

        switch( display) {
          case 0: break;
          default:
            disp->clrline();

            if ((tmp = vars.getVar("show_bots")) && atoi(tmp) && getName() && *getName())
              *disp << CWHITE << getName() << " ";

              *disp << CWHITE  << "["    << CBBLUE   << printTime()        << CWHITE << "] "
                    << CBWHITE << "*** " << CBPURPLE << cuser->getName()    << " "
                    << CBWHITE << " ["   << CBCYAN   << cuser->getLatency() << CBGREEN << "ms" << CBWHITE << "] "
                    << CBWHITE << " ["   << CBGREEN  << cuser->getProduct() << CBWHITE << "] "
                    << CBWHITE << " ["   << CBGREEN  << cuser->getFlags() << CBWHITE << "] "
                    << CBWHITE << " has joined the channel." << CWHITE << endl;

            input->update();
            break;
          }
      }


  // Is the user banned?
    // Command *cmd = commandlist.getCommand( getCommandSet());
    User *user = command->userlist->getUser( pkt_buf + 0x1C);
    if (user)
      if (user->getFlags() & 1) {
        enqueue( 1, "/ban %s %s", user->getName(), user->getBanMessage());
        return( 0);
      }

  // Are we in channel lockdown?
    if ((cmdMode == 1) && command) {
      User *user = command->userlist->getUser( cuser->getName());
      char buf[ 1024];
      if (user) {
        if (user->getAccess() < 20) {
          // sprintf( buf, "/ban %s (CHANNEL LOCKDOWN)", cuser->getName());
          enqueue( 4, "/ban %s (CHANNEL LOCKDOWN)", cuser->getName());
          return( 0);
        }
      } else {
        //sprintf( buf, "/ban %s (CHANNEL LOCKDOWN)", cuser->getName());
        enqueue( 4, "/ban %s (CHANNEL LOCKDOWN)", cuser->getName());
        return( 0);
      }
    }

  // Are they tagbanned?
    if (command) {
      TagBan *tagban = command->getTagBan();
      if (tagban)
        if (tagban->isBanned( cuser->getName())) {
          //char buf[ 1024];
          //sprintf( buf, "/ban %s (TAGBAN)", cuser->getName());
          enqueue( 4, "/ban %s (TAGBAN)", cuser->getName());
          return( 0);
        }
    }

  // Do we have an auto-welcome ?
    if (char *var = vars.getVar( "auto_welcome"))
      if (atoi( var)) {
        //char tmp[ 8192];
        //sprintf(tmp, "/msg %s Welcome to the channel %s. Type !help for help.", cuser->getName(), cuser->getName());
        enqueue( 7, "/msg %s Welcome to the channel %s. Type !help for help.\0__________________________________________________________________________________", cuser->getName(), cuser->getName());
      }

  return( 0);
}


int Bot::pkt_0f_03( int pkt_len, char *pkt_buf) {
  // SamuelJack has left the channel.
  //> 0000   FF 0F 2D 00 03 00 00 00 - 00 00 00 00 32 02 00 00   ..-.........2...
  //> 0010   00 00 00 00 0D F0 AD BA - 0D F0 AD BA 4C 65 67 65   ............Lege
  //> 0020   6E 64 60 49 6C 6C 75 6D - 69 6E 61 00 00            nd llumina..

  CUser *user = clist->getUser( pkt_buf + 0x1C);

  if (!user) {
    disp->clrline();
    *disp << "Debug: Received a channel leave by " << (pkt_buf + 0x1c) << "." << endl
          << "       We didn't know he was in the channel." << endl;
    input->update();
    return( 0);
  }
  
  disp->delUser( user->getName());
  //switch( display) {
  //  case 0: break;
  //  case 1:
      if (char *tmp = vars.getVar("show_joins"))
        if (atoi(tmp)) {
          disp->clrline();
          if ((tmp = vars.getVar("show_bots")) && atoi(tmp) && getName() && *getName())
            *disp << CWHITE << getName() << " ";
          *disp << CWHITE  << "[" << CBBLUE << printTime() << CWHITE << "] "
               << CBWHITE << "*** " << CBRED << (pkt_buf + 0x1C) << CBWHITE << " has left the channel." << CWHITE << endl;
          input->update();
        }
  //    break;
  //  case 2:
  //    window->add("*** has left the channel.\n");
  //    break;
  //}

  if (clist->delUser( user->getName())) {
    disp->clrline();
    *disp << "Debug: clist->delUser() returned an error." << endl;
    input->update();
  }

  return( 0);
}


int Bot::pkt_0f_04( int pkt_len, char *pkt_buf) {
  // Private message.
  //> 0000   FF 0F 47 00 04 00 00 00 - 00 00 00 00 82 07 00 00   ..G.............
  //> 0010   00 00 00 00 0D F0 AD BA - 0D F0 AD BA 44 61 72 6B   ............Dark
  //> 0020   7E 53 63 72 65 61 6D 00 - 74 68 61 6E 6B 20 79 6F   ~Scream.thank yo
  //> 0030   75 20 77 68 61 74 20 69 - 73 20 74 68 65 79 20 62   u what is they b
  //> 0040   69 64 64 69 6E 67 00    -                           idding.

    disp->clrline();
    char *tmp;
    if ((tmp = vars.getVar("show_bots")) && atoi(tmp) && getName() && *getName())
      *disp << CWHITE << getName() << " ";
    *disp << CWHITE  << "[" << CBBLUE << printTime() << CWHITE << "] ";
    
    if (char *tmp = vars.getVar("beep_onpriv"))
      if (atoi(tmp))
        printf("\a");
        
    *disp << CBCYAN << "[" << CBGREEN << (pkt_buf + 0x1C) << CBCYAN << "]" << CBWHITE << ": " 
          << CWHITE << (pkt_buf + 0x1D + strlen( pkt_buf + 0x1C)) << endl;
    input->update();

    if (Command *cmd = commandlist.getCommand( getCommandSet())) {
      cmd->handleCommand( this, pkt_buf + 0x1C, pkt_buf + 0x1D + strlen( pkt_buf + 0x1C), 1);
    } else {
      disp->clrline();
      *disp << "Error: " << getName() << " - no command handler was found" << endl;
      input->update();
    }

  return( 0);
}


int Bot::pkt_0f_05( int pkt_len, char *pkt_buf) {
  // PUBLIC MESSAGE
  // SamuelJack: I am really cool!
  //> 0000   FF 0F 37 00 05 00 00 00 - 00 00 00 00 1F 00 00 00   ..7.............
  //> 0010   00 00 00 00 0D F0 AD BA - 0D F0 AD BA 4A 6F 72 64   ............Jord
  //> 0020   61 6E 34 33 38 35 00 61 - 6C 74 61 72 20 61 6C 6C   an4385.altar all
  //> 0030   79 20 79 61 75 3F 00    -                           y yau?.

    //if (display == 2) {
    //  window->add((pkt_buf + 0x1C));
    //  window->add((pkt_buf + 0x1D + strlen( pkt_buf + 0x1C)));
    //  window->add("\n");
    //}

      disp->clrline();
    {
      char *tmp;

      if ((tmp = vars.getVar("show_bots")) && atoi(tmp) && getName() && *getName())
        *disp << CWHITE << getName() << " ";

      char *msg = pkt_buf + 0x1D + strlen( pkt_buf + 0x1C);
      char *cmsg = (char *)zenc_decrypt( (unsigned char *)msg);
      if (cmsg) {
              *disp << CWHITE  << "[" << CBBLUE << printTime() << CWHITE << "] "
                    << CBGREEN << (pkt_buf + 0x1C) << CBWHITE << ": "
                    << CBWHITE << "[" << CBRED << "zENC" << CBWHITE << "] "
                    << CWHITE << cmsg << endl;
      } else {
        switch(*(pkt_buf + 0x1D + strlen( pkt_buf + 0x1C))) {
          case '':
            *disp << CWHITE  << "[" << CBBLUE << printTime() << CWHITE << "] "
                  << CBGREEN << (pkt_buf + 0x1C) << CBWHITE << ": "
                  << CBWHITE << "[" << CBRED << "vL" << CBWHITE << "] "
                  << CWHITE  << Decrypt( pkt_buf + 0x1D + strlen( pkt_buf + 0x1C) + 1) << endl;
            break;
          case '':
            *disp << CWHITE  << "[" << CBBLUE << printTime() << CWHITE << "] "
                  << CBGREEN << (pkt_buf + 0x1C) << CBWHITE << ": "
                  << CBWHITE << "[" << CBRED << "HEX" << CBWHITE << "] "
                  << CWHITE  << hex_decrypt( pkt_buf + 0x1D + strlen( pkt_buf + 0x1C)) << endl;
            break;
          default:
            *disp << CWHITE  << "[" << CBBLUE << printTime() << CWHITE << "] "
                  << CBGREEN << (pkt_buf + 0x1C) << CBWHITE << ": "
                  << CWHITE << (pkt_buf + 0x1D + strlen( pkt_buf + 0x1C)) << endl;
        }
      }

      if (Command *cmd = commandlist.getCommand( getCommandSet())) {
        if (cmsg)
          cmd->handleCommand( this, pkt_buf + 0x1C, cmsg, 0);
        else
          cmd->handleCommand( this, pkt_buf + 0x1C, msg, 0);
      } else {
        *disp << "Error: " << getName() << " - no command handler was found" << endl;
      }
      // cmd_Handler( pkt_buf + 0x1C, pkt_buf + 0x1D + strlen( pkt_buf + 0x1C));

      //if (*(pkt_buf + 0x1D + strlen( pkt_buf + 0x1C)) == '') {
      //  char *p = Decrypt( pkt_buf + 0x1D + strlen( pkt_buf + 0x1C) + 1);
      //  disp->clrline();
      //  *disp << "DEBUG: Decrypted - " << p << endl;
      //  input->update();
      //}

      /*
      if (!strncmp( msg, "!profile ", 9) && !strcasecmp( pkt_buf + 0x1C, "Eurijk!#2")) {
        if (tmp = vars.getVar("cmd_profile"))
          if (atoi(tmp)) {
            char *name = msg + 9;
            if (char *tmp = strstr( name, " ")) *tmp = 0;
            if (strlen( tmp) > 32) tmp[32] = 0;
      
            if (strlen( name) && (time(0) - profile_lasttime) >= 5) {
              profile_lasttime = time( 0);
              if (sendReqProfile( name)) {
                disp->clrline();
                *disp << "Error: Unable to send profile request" << endl;
                input->update();
              } else {
                disp->clrline();
                *disp << "Info:  Profile request sent." << endl;
                input->update();
              }
            }
          }
      }
      */

      input->update();
    };
  return( 0);
}


int Bot::pkt_0f_07( int pkt_len, char *pkt_buf) {
  // Channel Join (by you)
  //   0000   FF 0F 3B 00 07 00 00 00 - 01 00 00 00 5E 00 00 00   ..;.........^...
  //   0010   00 00 00 00 0D F0 AD BA - 0D F0 AD BA 44 53 7E 42   ............DS~B
  //   0020   6F 74 21 00 57 61 72 32 - 20 4C 61 64 64 65 72 20   ot!.War2 Ladder
  //   0030   43 68 61 6C 6C 65 6E 67 - 65 73 00                  Challenges.

  // let's see what lastlag was
    if (lag_spoof) {
      int t = *(u32 *)(pkt_buf + 0x0c);
      if ((t >= 10) || (t < 0)) {
        disp->clrline();
        *disp << CBWHITE << "Info:  " << CWHITE << "Lag - is " << CBCYAN << t << CBWHITE << "ms "
                                      << CWHITE << "- was " << CBCYAN << lastlag << CBWHITE << "ms "
                                      << CWHITE << "- 0point " << CBCYAN << lastlag + t << CBWHITE << "ms" << endl;
        input->update();

        lastlag = lastlag + t;
        sendpkt( 0x10ff);
        sendpkt( 0x02ff);

        netclose();
        return( -1);
      }
    }


  // Main window update.
    #ifndef DISP_CON
      *getDisp() << CBWHITE << getName() << CWHITE << " has joined " << CBCYAN
                << (pkt_buf + 0x1c + strlen( pkt_buf + 0x1c) + 1) << CWHITE << endl;
    #endif

  // Update our /who listing.
    clist->clear();
    disp->clearUsers();

    clist->setChannel( pkt_buf + 0x1c + strlen( pkt_buf + 0x1c) + 1);
    proxy->setChan( pkt_buf + 0x1c + strlen( pkt_buf + 0x1c) + 1);

    disp->clrline();

    {
      char *tmp;
      if ((tmp = vars.getVar("show_bots")) && atoi(tmp) && getName() && *getName())
        *disp << CWHITE << getName() << " ";
    }
    char bob[32];
    sprintf( bob, "0x%.8x", *(u32 *)(pkt_buf + 0x08));
    *disp << CWHITE  << "[" << CBBLUE << printTime() << CWHITE << "] ";
    *disp << CBWHITE << "*** You have joined channel " << CBGREEN << (pkt_buf + 0x1c + strlen( pkt_buf + 0x1c) + 1) 
          << CWHITE << "  [flags: " << CCYAN << bob << CWHITE << "]" << endl;
    strncpy( curchannel, (pkt_buf + 0x1c + strlen( pkt_buf + 0x1c) + 1), sizeof( curchannel) -1);

    if (strcasecmp( pkt_buf + 0x1C + strlen( pkt_buf + 0x1C) + 1, homechannel))
      if (*homechannel) {
        // disp << "Debug: Sending channel join request " 
        // disp << "[" << (pkt_buf + 0x1c + strlen( pkt_buf + 0x1c) + 1) << "]  "
        //      << "[" << homechannel << "]" << endl;
        char buf[ 1024];

        //sprintf( buf, "/join %s", homechannel);
        if (atoi(vars.getVar("auto_rejoin")))
          enqueue( 4, "/join %s", homechannel);
      }
    input->update();

  return( 0);
}


int Bot::pkt_0f_09( int pkt_len, char *pkt_buf) {
  #ifdef DEBUGBOT
    disp->clrline();
    *disp << "DEBUGBOT: User flags changed (IGNOREING IGNORING)" << endl;
    input->update();
    return( 0);
  #endif

  // Todo: Ops status change.
  //  0000   FF 0F 26 00 09 00 00 00 - 02 00 00 00 0B 00 00 00   ..&.............
  //  0010   00 00 00 00 0D F0 AD BA - 0D F0 AD BA 44 53 7E 42   ............DS~B
  //  0020   6F 74 39 21 00 00       -                           ot9!..

  CUser *cuser = clist->getUser( pkt_buf + 0x1c);

  if (cuser) {
    cuser->setFlags( *(u32 *)(pkt_buf + 0x08));
    cuser->setLatency( *(s32 *)(pkt_buf + 0x0C));
  }

  // Handle the Display
    if (char *tmp = vars.getVar("show_joins"))
      if (atoi(tmp)) {
        if (tmp = vars.getVar("beep_onjoin"))
          if (atoi(tmp))
            printf("\a");

        switch( display) {
          case 0: break;
          case 1:
            break;
            disp->clrline();

            if ((tmp = vars.getVar("show_bots")) && atoi(tmp) && getName() && *getName())
              *disp << CWHITE << getName() << " ";

              *disp << CWHITE  << "["    << CBBLUE   << printTime()        << CWHITE << "] "
                    << CBWHITE << "*** " << CBPURPLE << cuser->getName()    << " "
                    << CBWHITE << " ["   << CBCYAN   << cuser->getLatency() << CBGREEN << "ms" << CBWHITE << "] "
                    << CBWHITE << " ["   << CBGREEN  << cuser->getProduct() << CBWHITE << "] "
                    << CBWHITE << " ["   << CBGREEN  << cuser->getFlags() << CBWHITE << "] "
                    << CBWHITE << " has changed status." << CWHITE << endl;

            input->update();
          case 2:
            break;
          }
      }



  return( 0);
}

int Bot::pkt_0f_0A( int pkt_len, char *pkt_buf) {
  // Message sent by you.
  //> 0000   FF 0F 48 00 0A 00 00 00 - 00 00 00 00 B5 01 00 00   ..H.............
  //> 0010   00 00 00 00 0D F0 AD BA - 0D F0 AD BA 2D 6C 29 72   ............-l)r
  //> 0020   61 67 6F 6E 2D 00 57 65 - 6C 63 6F 6D 65 20 74 6F   agon-.Welcome to
  //> 0030   20 74 68 65 20 63 68 61 - 6E 6E 65 6C 20 2D 6C 29    the channel -l)
  //> 0040   72 61 67 6F 6E 2D 2E 00 -                           ragon-..
  disp->clrline();
  *disp << "/msg " << (pkt_buf + 0x1C) << " " << (pkt_buf + 0x1D + strlen( pkt_buf + 0x1C)) << endl;
  input->update();

  return( 0);
}


int Bot::pkt_0f_12( int pkt_len, char *pkt_buf) {
  // System Message
  //   0000   FF 0F 45 00 12 00 00 00 - 10 00 00 00 EA 00 00 00   ..E.............
  //   0010   00 00 00 00 0D F0 AD BA - 0D F0 AD BA 44 53 7E 42   ............DS~B
  //   0020   6F 74 21 00 54 68 69 73 - 20 73 65 72 76 65 72 20   ot!.This server
  //   0030   69 73 20 68 6F 73 74 65 - 64 20 62 79 20 45 78 6F   is hosted by Exo
  //   0040   64 75 73 2E 00          -                           dus..

    disp->clrline();
    { char *tmp;
    if ((tmp = vars.getVar("show_bots")) && atoi(tmp) && getName() && *getName())
      *disp << CWHITE << getName() << " ";
    }
    *disp << CWHITE  << "[" << CBBLUE << printTime() << CWHITE << "] ";
    *disp << CBWHITE << "*** " << CWHITE << (pkt_buf + 0x1c + strlen( pkt_buf + 0x1c) + 1) << CWHITE << endl;
    input->update();

  return( 0);
}


int Bot::pkt_0f_13( int pkt_len, char *pkt_buf) {
  // 0000   FF 0F 41 00 13 00 00 00 - 00 00 00 00 06 00 00 00   ..A.............
  // 0010   00 00 00 00 0D F0 AD BA - 0D F0 AD BA 44 53 7E 42   ............DS~B
  // 0020   6F 74 39 21 00 54 68 61 - 74 20 75 73 65 72 20 69   ot9!.That user i
  // 0030   73 20 6E 6F 74 20 6C 6F - 67 67 65 64 20 6F 6E 2E   s not logged on.
  // 0040   00                      -                           .

    disp->clrline();
    { char *tmp;
    if ((tmp = vars.getVar("show_bots")) && atoi(tmp) && getName() && *getName())
      *disp << CWHITE << getName() << " ";
    }
    *disp << CWHITE  << "[" << CBBLUE << printTime() << CWHITE << "] ";
    *disp << CBWHITE << "*** " << CBRED << (pkt_buf + 0x1c + strlen( pkt_buf + 0x1c) + 1) << CWHITE << endl;
    input->update();

  return( 0);
}

int Bot::pkt_0f_17( int pkt_len, char *pkt_buf) {
  // ACTION user looks around
  //> 0000   FF 0F 31 00 17 00 00 00 - 10 00 00 00 CB 00 00 00 ..1.............
  //> 0010   00 00 00 00 0D F0 AD BA - 0D F0 AD BA 44 53 7E 42 ............DS~B
  //> 0020   6F 74 21 00 6C 6F 6F 6B - 73 20 61 72 6F 75 6E 64   ot!.looks around
  //> 0030   00                      -                           .

      disp->clrline();
      { char *tmp;
        if ((tmp = vars.getVar("show_idles")) && !atoi(tmp)) {
          // Damn shaddup! Geezus! >;-)
          //if (strncmp( (pkt_buf + 0x1D + strlen( pkt_buf + 0x1C)), "is a ", 5))
          //  if (strncmp( (pkt_buf + 0x1D + strlen( pkt_buf + 0x1C)), "is an ", 6))
          //    if (strncmp( (pkt_buf + 0x1D + strlen( pkt_buf + 0x1C)), "NuLLKLoCk: ", 11)) 
          //      if (strncmp( (pkt_buf + 0x1D + strlen( pkt_buf + 0x1C)), "-=^un4^=-: ", 11)) 
          //        if (strncmp( (pkt_buf + 0x1D + strlen( pkt_buf + 0x1C)), "AutoHeal is a ", 14))
          //          if (strncmp( (pkt_buf + 0x1D + strlen( pkt_buf + 0x1C)), "vLBBot - a ", 11))
          //            if (strncmp( (pkt_buf + 0x1D + strlen( pkt_buf + 0x1C)), "~The Gate |{eepers~ -- ", 23))
          //              if (strcmp( (pkt_buf + 0x1D + strlen( pkt_buf + 0x1C)), "  . dr  zL"))
          //                if (strcmp( (pkt_buf + 0x1D + strlen( pkt_buf + 0x1C)), "b. lz v.  b z"))
                            if (strcmp( (pkt_buf + 0x1D + strlen( pkt_buf + 0x1C)), "- Chewbacca Bot v2.0a2 - anti-idle")) {
          if ((tmp = vars.getVar("show_bots")) && atoi(tmp) && getName() && *getName())
            *disp << CWHITE << getName() << " ";

          *disp << CWHITE  << "[" << CBBLUE << printTime() << CWHITE << "] ";
          *disp << CBYELLOW << (pkt_buf + 0x1C) << " " << (pkt_buf + 0x1D + strlen( pkt_buf + 0x1C)) << CWHITE << endl;
                          }
        } else {
          if ((tmp = vars.getVar("show_bots")) && atoi(tmp) && getName() && *getName())
            *disp << CWHITE << getName() << " ";

          *disp << CWHITE  << "[" << CBBLUE << printTime() << CWHITE << "] ";
          *disp << CBYELLOW << (pkt_buf + 0x1C) << " " << (pkt_buf + 0x1D + strlen( pkt_buf + 0x1C)) << CWHITE << endl;
        }
      }
      input->update();

    if (Command *cmd = commandlist.getCommand( getCommandSet())) {
      cmd->handleCommand( this, pkt_buf + 0x1C, pkt_buf + 0x1D + strlen( pkt_buf + 0x1C), 0);
    } else {
      disp->clrline();
      *disp << "Error: " << getName() << " - no command handler was found" << endl;
      input->update();
    }

  return( 0);
}


int Bot::pkt_0f( int pkt_len, char *pkt_buf) {
  if (pkt_len < 6)
    return( -1);

  switch( *(u16 *)(pkt_buf + 4)) {
    case 0x01: return( pkt_0f_01( pkt_len, pkt_buf));
    case 0x02: return( pkt_0f_02( pkt_len, pkt_buf));
    case 0x03: return( pkt_0f_03( pkt_len, pkt_buf));
    case 0x04: return( pkt_0f_04( pkt_len, pkt_buf));
    case 0x05: return( pkt_0f_05( pkt_len, pkt_buf));
    case 0x07: return( pkt_0f_07( pkt_len, pkt_buf));
    case 0x09: return( pkt_0f_09( pkt_len, pkt_buf));
    case 0x0A: return( pkt_0f_0A( pkt_len, pkt_buf));
    case 0x12: return( pkt_0f_12( pkt_len, pkt_buf));
    case 0x13: return( pkt_0f_13( pkt_len, pkt_buf));
    case 0x17: return( pkt_0f_17( pkt_len, pkt_buf));
    default:
      disp->clrline();
      *disp << "Info:  *** Received from server ***" << endl;
      disp->hexdump( pkt_len, pkt_buf);
      input->update();
  }

  return( 0);
}


int Bot::pkt_15( int len, char *buf) {
  if (char *tmp = vars.getVar("debug_idle"))
    if (atoi( tmp)) {
      disp->clrline();
      *disp << CBCYAN << "Info:  " << CBWHITE << "Received an add from the server." << CWHITE << endl;
      // hexdump( len, buf, stdout, 1);
      // send( cfd, buf, len, 0);
      input->update();
    }
  lastadd_num = *(u32 *)(buf + 4);

  // === Let the server know we went out and downloaded the add *cough* BS ===
    u32 tmp[ 5] = { 0x001221FF, 0x49583836, 0x5732424E, lastadd_num, 0 };
    switch( getClientType()) {
      case 2: // RTSJ
        tmp[3] = 0x4A535452;
        break;
      case 1:
        tmp[3] = 0x53544152;
        break;
      default:
        break;
    }
    rawsend( 0x12, (char *)tmp);

  return( 0);
}


int Bot::pkt_19( int pkt_len, char *pkt_buf) {
  // 0000   FF 19 3C 00 10 00 00 00 - 59 6F 75 20 68 61 76 65   ..<.....You have
  // 0010   20 62 65 65 6E 20 64 69 - 73 63 6F 6E 6E 65 63 74    been disconnect
  // 0020   65 64 20 66 6F 72 20 66 - 6C 6F 6F 64 69 6E 67 2E   ed for flooding.
  // 0030   00 42 61 74 74 6C 65 2E - 6E 65 74 00               .Battle.net.
  disp->clrline();
  *disp << "*** " << (pkt_buf + 8) << endl;
  input->update();
  netclose();
  return( 0);
}


int Bot::pkt_1d( int pkt_len, char *pkt_buf) {
  
  if (pkt_len != 0x0c) {
    disp->clrline();
    *disp << "Error: Packet 0x1dff was " << pkt_len << " bytes long." << endl
         << "       Something is suspicious jim." << endl;
    disp->hexdump( pkt_len, pkt_buf);
    input->update();
    netclose();
    return( -1);
  }
  
  session_num = *(u32 *)(pkt_buf + 4);
  session_key = *(u32 *)(pkt_buf + 8);

  #ifdef DEBUGBOT
    disp->clrline();
    *disp << "Info:  [0x1dff] Session Number: " << session_num << " Session_key: " << session_key << endl
          << "       Sending [0x12ff] CLIENT_COUNTRYINFO" << endl;
    input->update();
  #endif

  // bnetd time stuffs
  {
        double secs = time(0) + 11644473600.0;
        secs += 3600 * 8; // heh
        *this << (u32)(secs*.0000001)
            << (u32)(secs*.00232830643653869628);
        secs -= 3600 * 8; // heh

        // Same for WC2 and SC
        *this << (u32)(secs*.0000001)
            << (u32)(secs*.00232830643653869628)
            << (u32)0          // time bias - TODO: blah whatever >;-) hope they don't care
            << (u32)0x00000409 // english
            << (u32)0x00000409 // english
            << (u32)0x00000409 // english
            << "enu"
            << "1"
            << "USA"
            << "United States";
        sendpkt( 0x12ff);
      }
      //mine
      //  < 0000   FF 12 3C 00 EF 04 00 00 - CE F9 C0 01 EF 04 00 00   ..<.............
      //  < 0010   8B F9 C0 01 00 00 00 00 - 09 04 00 00 09 04 00 00   ................
      //  < 0020   09 04 00 00 65 6E 75 00 - 31 00 55 53 41 00 55 6E   ....enu.1.USA.Un
      //  < 0030   69 74 65 64 20 53 74 61 - 74 65 73 00               ited States.
      //correct
      //    0000   FF 12 3C 00 C0 D7 42 DF - 7A F9 C0 01 C0 FF E3 32
      //    0010   40 F9 C0 01 E0 01 00 00 - 09 04 00 00 09 04 00 00
      //    0020   09 04 00 00 65 6E 75 00 - 31 00 55 53 41 00 55 6E   ....enu.1.USA.United
      //    0030   69 74 65 64 20 53 74 61 - 74 65 73 00 
      //correct
      //< 0050     FF 06 14 00 36 38 58 49 - 4E 42 32 57 4F 00 00 00
      //           00 00 00 00
      //mine
      //< 0000     FF 06 14 00 36 38 58 49 - 4E 42 32 57 4F 00 00 00   ....68XINB2WO...
      //< 0010     00 00 00 00             -                           ....

      #ifdef DEBUGBOT
        disp->clrline();
        *disp << "       Sending [0x06ff] CLIENT_PROGIDENT" << endl;
        input->update();
      #endif

      switch( getClientType()) {
        case 0:
          *this << (u32)0x49583836 //"68XINB2WO"
                << (u32)0x5732424e //(u16)0
                << (u32)0x0000004f //;
                << (u32)0;
          sendpkt( 0x06ff);
          break;
        case 1:
          *this << (u32)0x49583836 // 68XI
                << (u32)0x53544152 // RATS
                << (u32)0x000000c3 // ?
                << (u32)0;
          sendpkt( 0x06ff);
          break;
        case 2:
          *this << (u32)0x49583836 // 68XI
                << (u32)0x4A535452 // 
                << (u32)0x000000a9 // ?
                << (u32)0;
          sendpkt( 0x06ff);
          break;
      }

  return( 0);
}



int Bot::pkt_25( int pkt_len, char *pkt_buf) {
  //disp->clrline();
  //*disp << CWHITE  << "[" << CBBLUE << printTime() << CWHITE << "] ";
  //*disp << CBWHITE << "*** " << CBRED << "PING " << *(u32 *)(pkt_buf + 4) << CWHITE << endl;
  //input->update();
  lastping = time( 0);

  if (lag_spoof)
    *(u32 *)(pkt_buf + 4) += lastlag;

  rawsend( pkt_len, pkt_buf);
  
  return( 0);
}

void Bot::showProfile( char *txt, char *msg) {
  *disp << CBGREEN << txt << CBCYAN;

  while (*msg) {
    switch (*msg) {
      case '\r': break;
      case '\n': *disp << endl << "          ";
                 break;
      default:
        if (*msg >= 32)
          *disp << *msg;
    }
    msg++;
  }
  *disp << CWHITE << endl;
}


// TODO: Make sure server data is valid.
int Bot::pkt_26( int pkt_len, char *pkt_buf) {
  disp->clrline();
  *disp << CBWHITE << "Info:  " << CWHITE << "Received a profile packet." << endl;
  // disp->hexdump( pkt_len, pkt_buf);
  char *tmp = pkt_buf + 0x10;
 
  showProfile( "Sex:      ", tmp); tmp += strlen( tmp) + 1;
  showProfile( "Age:      ", tmp); tmp += strlen( tmp) + 1;
  showProfile( "Location: ", tmp); tmp += strlen( tmp) + 1;
  showProfile( "Desc:     ", tmp);

  //*disp << CBGREEN << "Sex:      " << CBCYAN << tmp << endl; tmp += strlen( tmp) + 1;
  //if (t = strchr( tmp, '\r')) *t = ' '; if (t = strchr( tmp, '\n')) *t = ' ';
  //*disp << CBGREEN << "Age:      " << CBCYAN << tmp << endl; tmp += strlen( tmp) + 1;
  //if (t = strchr( tmp, '\r')) *t = ' '; if (t = strchr( tmp, '\n')) *t = ' ';
  //*disp << CBGREEN << "Location: " << CBCYAN << tmp << endl; tmp += strlen( tmp) + 1;
  //if (t = strchr( tmp, '\r')) *t = ' '; if (t = strchr( tmp, '\n')) *t = ' ';
  //*disp << CBGREEN << "Desc:     " << CBCYAN << tmp << endl;
  input->update();

  return( 0);
}


int Bot::pkt_29( int pkt_len, char *pkt_buf) {
  // FF 29 08 00 01 00 00 00

  #ifdef DEBUGBOT
    disp->clrline();
    *disp << "Info:  [0x29ff] Received user/pass response" << endl;
    input->update();
  #endif

  if (*(u16 *)(pkt_buf + 2) != 8) {
    disp->clrline();
    *disp << "Error: Invalid password packet received. Aborting." << endl;
    input->update();
    netclose();
    return( -1);
  }

  if (*(u32 *)(pkt_buf + 4) != 1) {
    disp->clrline();
    *disp << "Error: Invalid password. Creating account." << endl;
    input->update();

    char _password[256];
    strcpy( _password, password);
    u32 hash[ 5];

    HashData( _password, strlen( _password), (char *)hash);
    *this << hash[0] << hash[1] << hash[2] << hash[3] << hash[4] << getName();
    sendpkt( 0x02AFF);

    // netclose();
    return( -1);
  }

  #ifdef DEBUGBOT
    disp->clrline();
    *disp << "       Password was valid." << endl
          << "       Sending [0x0aff] Realm login" << endl;
    input->update();
  #endif

  *this << username
        << "";
  sendpkt( 0x0aff);

  #ifdef DEBUGBOT
    disp->clrline();
    *disp << "       Sending [0x0bff] Prog Ident" << endl;
    input->update();
  #endif

  switch( getClientType()) {
    case 0:
      *this << (u32)0x5732424E; // "NB2W"
      sendpkt( 0x0bff);
      break;
    case 1:
      *this << (u32)0x53544152; // "RATS"
      sendpkt( 0x0bff);
      break;
    case 2:
      *this << (u32)0x4A535452; // "japthingy"
      sendpkt( 0x0bff);
      break;
  }

  #ifdef DEBUGBOT
    disp->clrline();
    *disp << "       Sending [0x0cff] Channel Join" << endl;
    input->update();
  #endif

  msg_lasttime = time(0);
  char *bob = "The Void";
  char *tmp = getHomeChannel();
  if (!tmp) tmp = bob;
  //switch( getClientType()) {
  //  case 0: {
  //    if (!tmp) tmp = bob;
  *this << (u32)2
        << tmp; // War2BNE USA-"; // << "war2bne";
  sendpkt( 0x0cff);
  //    }
  //    break;
  //  case 1:
  //    *this << (u32)1
  //          << tmp; // Starcraft USA-";
  //    sendpkt( 0x0cff);
  //    break;
  //  case 2:
  //    *this << (u32)1
  //          << tmp; //Starcraft USA-";
  //    sendpkt( 0x0cff);
  //    break;
  //}

  return( 0);
}

int Bot::pkt_2a( int pkt_len, char *pkt_buf) {
  // 0000   FF 2A 18 00 01 00 00 00 - 13 00 00 00 78 52 82 02   .*..........xR..
  // 0010   00 00 00 00 00 00 00 00 -                           ........
  netclose();
  return( 0);
}

int Bot::pkt_2d( int pkt_len, char *pkt_buf) {
  //< 0000   FF 2D 16 00 80 10 FD 1D - 38 C4 BF 01 69 63 6F 6E   .-......8...icon
  //< 0010   73 2E 62 6E 69 00       -                           s.bni.

  #ifdef DEBUGBOT
    disp->clrline();
    *disp << "Info:  [0x2dff] Received Icon Reply" << endl;
    input->update();
  #endif

  return( 0);
}

int Bot::pkt_33( int pkt_len, char *pkt_buf) {
  #ifdef DEBUGBOT
    disp->clrline();
    *disp << "Info:  [0x33ff] Received a file timestamp info packet" << endl;
    input->update();
  #endif

    switch (getClientType()) {
      case 3:
        #ifdef DEBUGBOT
        disp->clrline();
        *disp << "       " << CWHITE << " Sending [0x3aff] Username & Password" << endl;
        input->update();
        #endif

        {
        char *_username = getName();
        char _password[256];

	strcpy( _password, password);

	char lpResult[ 1024], lpResult2[ 1024];
	HashData( _password, strlen( _password), lpResult);
	u32 seed = rand();

	*(u32 *)(lpResult2 + 0) = seed;
	*(u32 *)(lpResult2 + 4) = session_key;
	memcpy( lpResult2 + 8, lpResult, 4 * 5);
	HashData( lpResult2, 4 * 7, lpResult);
        *this << (u32)seed
              << (u32)session_key
              << *(u32 *)(lpResult + 0)
              << *(u32 *)(lpResult + 4)
              << *(u32 *)(lpResult + 8)
              << *(u32 *)(lpResult + 12)
              << *(u32 *)(lpResult + 16)
              << _username;
        sendpkt( 0x3aff);
        }
        break;
      default:
        break;
    }

  return( 0);
}

int Bot::pkt_36( int pkt_len, char *pkt_buf) {
  //< 0000   FF 36 09 00 02 00 00 00 - 00                        .6.......

  #ifdef DEBUGBOT
    disp->clrline();
    *disp << "Info:  [0x36ff] Server CD reply" << endl;
    input->update();
  #endif

  if (*(u16 *)(pkt_buf + 2) < 0x09) {
    disp->clrline();
    *disp << "Error: Packet [0x36ff] was too short." << endl;
    disp->hexdump( pkt_len, pkt_buf);
    input->update();
    netclose();
    return( -1);
  }

  switch (*(u32 *)(pkt_buf + 4)) {
    case 1:
      #ifdef DEBUGBOT
        disp->clrline();
        *disp << "       CD Key Checked Ok" << endl;
        input->update();
      #endif

      switch( getClientType()) {
        case 0: // WarCraft
          #ifdef DEBUGBOT
            disp->clrline();
            *disp << "       Sending [0x33ff] File info request. (tos_USA.txt)" << endl;
            input->update();
          #endif

          *this << (u32)0x1A // first request
                << (u32)0
                << "tos_USA.txt";
          sendpkt( 0x33ff);

          #ifdef DEBUGBOT
            disp->clrline();
            *disp << "       Sending [0x33ff] File info request. (bnserver.ini)" << endl;
            input->update();
          #endif

          *this << (u32)0x1B // second request
                << (u32)0
                << "bnserver.ini";
          sendpkt( 0x33ff);

          #ifdef DEBUGBOT
            disp->clrline();
            *disp << "       Sending [0x26ff] Profile request (yeah right!)" << endl;
            input->update();
          #endif

          // Sending username/password hash.
          //   0000   FF 29 29 00 9A 63 49 35 - 70 33 D9 CF ?? ?? ?? ??   .))..cI5p3..????
          //   0010   ?? ?? ?? ?? ?? ?? ?? ?? - ?? ?? ?? ?? ?? ?? ?? ??   ????????????????
          //   0020   44 53 7E 42 6F 74 36 21 - 00                        DS~Bot6!.

          #ifdef DEBUGBOT
            disp->clrline();
            *disp << "       Sending [0x29ff] Username & password hash" << endl;
            input->update();
          #endif

          {
            char lpResult[ 1024], lpResult2[ 1024];
            HashData( password, strlen( password), lpResult);
            u32 seed = rand();
            *(u32 *)(lpResult2 + 0) = seed;
            *(u32 *)(lpResult2 + 4) = session_key;
            memcpy( lpResult2 + 8, lpResult, 4 * 5);
            HashData( lpResult2, 4 * 7, lpResult);
            *this << (u32)seed
                  << (u32)session_key
                  << *(u32 *)(lpResult + 0)
                  << *(u32 *)(lpResult + 4)
                  << *(u32 *)(lpResult + 8)
                  << *(u32 *)(lpResult + 12)
                  << *(u32 *)(lpResult + 16)
                  << username;
            sendpkt( 0x29ff);
          }
          break;
        case 1: // StarCraft
          #ifdef DEBUGBOT
            disp->clrline();
            *disp << "       Sending [0x33ff] File info request. (icons_STAR.bni)" << endl;
            input->update();
          #endif

          *this << (u32)0x1A // first request
                << (u32)0
                << "icons_STAR.bni";
          sendpkt( 0x33ff);

          #ifdef DEBUGBOT
            disp->clrline();
            *disp << "       Sending [0x33ff] File info request. (tos_USA.txt)" << endl;
            input->update();
          #endif

          *this << (u32)0x1B // first request
                << (u32)0
                << "tos_USA.txt";
          sendpkt( 0x33ff);


          #ifdef DEBUGBOT
            disp->clrline();
            *disp << "       Sending [0x33ff] File info request. (bnserver.ini)" << endl;
            input->update();
          #endif

          *this << (u32)0x1C // second request
                << (u32)0
                << "bnserver.ini";
          sendpkt( 0x33ff);

          #ifdef DEBUGBOT
            disp->clrline();
            *disp << "       Sending [0x26ff] Profile request (yeah right!)" << endl;
            input->update();
          #endif

          #ifdef DEBUGBOT
            disp->clrline();
            *disp << "       Sending [0x29ff] Username & password hash" << endl;
            input->update();
          #endif

          {
            char lpResult[ 1024], lpResult2[ 1024];
            HashData( password, strlen( password), lpResult);
            u32 seed = rand();
            *(u32 *)(lpResult2 + 0) = seed;
            *(u32 *)(lpResult2 + 4) = session_key;
            memcpy( lpResult2 + 8, lpResult, 4 * 5);
            HashData( lpResult2, 4 * 7, lpResult);
            *this << (u32)seed
                  << (u32)session_key
                  << *(u32 *)(lpResult + 0)
                  << *(u32 *)(lpResult + 4)
                  << *(u32 *)(lpResult + 8)
                  << *(u32 *)(lpResult + 12)
                  << *(u32 *)(lpResult + 16)
                  << username;
            sendpkt( 0x29ff);
          }
          break;
        case 2: // Jap Starcraft
          #ifdef DEBUGBOT
            disp->clrline();
            *disp << "       Sending [0x33ff] File info request. (bnserver.ini)" << endl;
            input->update();
          #endif

          *this << (u32)0x1A // first request
                << (u32)0
                << "bnserver.ini";
          sendpkt( 0x33ff);

          #ifdef DEBUGBOT
            disp->clrline();
            *disp << "       Sending [0x26ff] Profile request (yeah right!)" << endl;
            input->update();
          #endif

          #ifdef DEBUGBOT
            disp->clrline();
            *disp << "       Sending [0x29ff] Username & password hash" << endl;
            input->update();
          #endif

          {
            char lpResult[ 1024], lpResult2[ 1024];
            HashData( password, strlen( password), lpResult);
            u32 seed = rand();
            *(u32 *)(lpResult2 + 0) = seed;
            *(u32 *)(lpResult2 + 4) = session_key;
            memcpy( lpResult2 + 8, lpResult, 4 * 5);
            HashData( lpResult2, 4 * 7, lpResult);
            *this << (u32)seed
                  << (u32)session_key
                  << *(u32 *)(lpResult + 0)
                  << *(u32 *)(lpResult + 4)
                  << *(u32 *)(lpResult + 8)
                  << *(u32 *)(lpResult + 12)
                  << *(u32 *)(lpResult + 16)
                  << username;
            sendpkt( 0x29ff);
          }
          break;
      }
      break;
    case 0x00000002:
          disp->clrline();
          *disp << "Error: Invalid CD Key returned." << endl;
          input->update();
          netclose();
          break;
    case 0x00000003:
          disp->clrline();
          *disp << "Error: CD Key is not for this application." << endl
                << "       Did you enter a StarCraft CD Key for a Warcraft setting?" << endl;
          input->update();
          netclose();
          break;
    case 0x00000004:
          disp->clrline();
          *disp << "Error: We got an error response. Strange, shutting down." << endl;
          input->update();
          netclose();
          break;
    case 0x00000005:
          disp->clrline();
          *disp << "Error: Your CD Key is currently in use." << endl;
          input->update();
          netclose();
          break;
    default:
          disp->clrline();
          *disp << "Error: We got an unknown response. Strange, shutting down." << endl;
          input->update();
          netclose();
          break;
    }

  #ifdef DEBUGBOT
    disp->clrline();
    *disp << "DEBUGBOT: Bot::pkt_36 has exited." << endl;
    input->update();
  #endif

  return( 0);
}


int Bot::pkt_3a( int pkt_len, char *pkt_buf) {
  // 0000   FF 3A 08 00 02 00 00 00

  switch( *(u32 *)(pkt_buf + 4)) {
    case 0:
      #ifdef DEBUGBOT
        disp->clrline();
        *disp << "Info:  [0x3aff] Username & Password OK" << endl
             << "       Sending [0x0bff] Channel request for D2XP" << endl
             << "       Sending [0x0aff] Character registration" << endl
             << "       Sending [0x0cff] Channel join" << endl;
        input->update();
      #endif
      break;
    default:
      disp->clrline();
      *disp << "Info:  [0x3aff] Username & Password" << endl
           << "Error: Did *NOT* receive OK. Closing connection." << endl;
      input->update();
      netclose();
      return( -1);
  }

  *this << (u32)0x44325850; // PX2D
  sendpkt( 0x0bff);

  char buf[ 1024];
  sprintf( buf, ",%s", nick);
  *this << nick
        << buf;
  sendpkt( 0x0aff);

  *this << (u32)1
        << "Diablo II";
  sendpkt( 0x0cff);

  return( 0);
}


int Bot::pkt_50( int pkt_len, char *pkt_buf) {
  // This assumes the remote server is not hostile.

  session_key = *(u32 *)(pkt_buf + 8);
  session_num = *(u32 *)(pkt_buf + 12);

  {
    char *hash_mpq = (pkt_buf + 0x18);
    char *hash_equ = (pkt_buf + strlen( hash_mpq) + 0x19);

    #ifdef DEBUGBOT
      disp->clrline();
      *disp << "Info:  [0x50ff] session_num: " << session_num << endl
            << "       session_key: " << session_key << endl
            << "       hash_mpq:    " << hash_mpq << endl
            << "       hash_equ:    " << hash_equ << endl;
      input->update();
    #endif

    #ifndef WIN32
    char *tmp = SendCDKeyCheckD2Exp( cdkey, cdkey2, session_key, getName(), rand(), hash_equ, hash_mpq);
    if (tmp) {
      #ifdef DEBUGBOT 
        disp->clrline();
        disp->hexdump( 0x59 + 8 + 18 + 8 + 8 + strlen( getName()), tmp);
        input->update();
      #endif
      rawsend( 0x59 + 8 + 18 + 8 + 8 + strlen( getName()), tmp);
    } else {
      disp->clrline();
      *disp << "Error: Unable to decode CD-Keys" << endl;
      input->update();
      netclose();
    }
    #endif
  }
  
  return( 0);
}


int Bot::pkt_51( int pkt_len, char *pkt_buf) {
  // 0000  ff 51 09 00 00 00 00 00 - 00
  switch( *(u32 *)(pkt_buf + 4)) {
    case 0:
      disp->clrline();
      *disp << "Info:  [0x51ff] CDKeys & Client Revision OK" << endl
           << "       Sending [0x33ff] File request: bnserver-D2DV.ini" << endl;
      input->update();
      *this << (u32)0x80000004 // 0x80 for "NEW INFO" ? File download request number
            << (u32)0
            << "bnserver-D2DV.ini";
      sendpkt( 0x33ff);
      break;
    default:
      disp->clrline();
      *disp << "Info:  [0x51ff] CDKeys & Client Revision" << endl
           << "Error: Did *NOT* receive OK. Closing connection." << endl;
      input->update();
      netclose();
  }
  return( 0);
}


int Bot::pkt_handler( int pkt_len, char *pkt_buf) {
  #ifdef DEBUGPKT
    disp->clrline();
    *disp << "Debug: Bot::pkt_handler was called" << endl;
    disp->hexdump( pkt_len, pkt_buf);
    input->update();
  #endif

  //if (!filter && (pkt_len >= 2))
  //  if (*(short int *)pkt_buf == 0x0fFF) {
  //    lastadd_time = time( 0);
  //    filter++;
  //    disp.clrline();
  //    disp << "Info:  *** Filter has been turned on ***" << endl;
  //    input.update();
  //  }//

  //if (!filter) {
  //  disp.clrline();
  //  disp << "Info:  *** No filter - Received from server ***" << endl;
  //  disp.hexdump( pkt_len, pkt_buf);
  //  input.update();
  //
  //  return( 0);
  //}

  switch( pkt_buf[1]) {
    case 0x00: return( pkt_00( pkt_len, pkt_buf));
    case 0x04: return( pkt_04( pkt_len, pkt_buf));
    case 0x05: return( pkt_05( pkt_len, pkt_buf));
    case 0x06: return( pkt_06( pkt_len, pkt_buf));
    case 0x07: return( pkt_07( pkt_len, pkt_buf));
    case 0x0a: return( pkt_0a( pkt_len, pkt_buf));
    case 0x0b: return( pkt_0b( pkt_len, pkt_buf));
    case 0x0c: return( pkt_0c( pkt_len, pkt_buf));
    case 0x0f: return( pkt_0f( pkt_len, pkt_buf));
    case 0x15: return( pkt_15( pkt_len, pkt_buf));
    case 0x19: return( pkt_19( pkt_len, pkt_buf));
    case 0x1d: return( pkt_1d( pkt_len, pkt_buf));
    case 0x25: return( pkt_25( pkt_len, pkt_buf));
    case 0x26: return( pkt_26( pkt_len, pkt_buf));
    case 0x29: return( pkt_29( pkt_len, pkt_buf));
    case 0x2a: return( pkt_2a( pkt_len, pkt_buf));
    case 0x2d: return( pkt_2d( pkt_len, pkt_buf));
    case 0x33: return( pkt_33( pkt_len, pkt_buf));
    case 0x36: return( pkt_36( pkt_len, pkt_buf));
    case 0x3a: return( pkt_3a( pkt_len, pkt_buf));
    case 0x50: return( pkt_50( pkt_len, pkt_buf));
    case 0x51: return( pkt_51( pkt_len, pkt_buf));
    default:
     disp->clrline();
     *disp << CBWHITE << "Info:  " << CBCYAN << getName() << CWHITE << " has received an unknown packet from the server." << endl;
     disp->hexdump( pkt_len, pkt_buf);
     input->update();
     netclose();

     #ifdef DEBUGBOT
       exit( -1);
     #endif

     break;
  }

  #ifdef DEBUGPKT
    disp->clrline();
    *disp << "Debug: Bot::pkt_handler is exiting" << endl;
    input->update();
  #endif

  return( 0);
}

//int Bot::enqueue( int len, char *buf, int lvl) {
//  //if (!filter)
//  //  return( -1);
//  if (getfd() == -1)
//    return( -1);
//  
//  rawsend( len, buf);
//}

int Bot::sendReqProfile( char *name) {
  char buf[ 1024];

  if (getfd() == -1) return( -1);
  //if (!profile_lastval) return( -1);
  //if (!profile_lasttime) return( -1);

    //char fuckyou[] = {
    //                                                  0x70, 0x72, 0x6F,   //Gay'n'Angry!.pro
    //  0x66, 0x69, 0x6C, 0x65, 0x5C, 0x73, 0x65, 0x78, 0x00, 0x70, 0x72, 0x6F, 0x66, 0x69, 0x6C, 0x65,   //file\sex.profile
    //  0x5C, 0x61, 0x67, 0x65, 0x00, 0x70, 0x72, 0x6F, 0x66, 0x69, 0x6C, 0x65, 0x5C, 0x6C, 0x6F, 0x63,   //\age.profile\loc
    //  0x61, 0x74, 0x69, 0x6F, 0x6E, 0x00, 0x70, 0x72, 0x6F, 0x66, 0x69, 0x6C, 0x65, 0x5C, 0x64, 0x65,   //ation.profile\de
    //  0x73, 0x63, 0x72, 0x69, 0x70, 0x74, 0x69, 0x6F, 0x6E, 0x00, 0x52, 0x65, 0x63, 0x6F, 0x72, 0x64,   //scription.Record
    //  0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C, 0x30, 0x5C, 0x77, 0x69, 0x6E, 0x73, 0x00, 0x52, 0x65, 0x63,   //\W2BN\0\wins.Rec
    //  0x6F, 0x72, 0x64, 0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C, 0x30, 0x5C, 0x6C, 0x6F, 0x73, 0x73, 0x65,   //ord\W2BN\0\losse
    //  0x73, 0x00, 0x52, 0x65, 0x63, 0x6F, 0x72, 0x64, 0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C, 0x30, 0x5C,   //s.Record\W2BN\0\
    //  0x64, 0x69, 0x73, 0x63, 0x6F, 0x6E, 0x6E, 0x65, 0x63, 0x74, 0x73, 0x00, 0x52, 0x65, 0x63, 0x6F,   //disconnects.Reco
    //  0x72, 0x64, 0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C, 0x30, 0x5C, 0x6C, 0x61, 0x73, 0x74, 0x20, 0x67,   //rd\W2BN\0\last g
    //  0x61, 0x6D, 0x65, 0x00, 0x52, 0x65, 0x63, 0x6F, 0x72, 0x64, 0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C,   //ame.Record\W2BN\
    //  0x30, 0x5C, 0x6C, 0x61, 0x73, 0x74, 0x20, 0x67, 0x61, 0x6D, 0x65, 0x20, 0x72, 0x65, 0x73, 0x75,   //0\last game resu
    //  0x6C, 0x74, 0x00, 0x52, 0x65, 0x63, 0x6F, 0x72, 0x64, 0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C, 0x31,   //lt.Record\W2BN\1
    //  0x5C, 0x77, 0x69, 0x6E, 0x73, 0x00, 0x52, 0x65, 0x63, 0x6F, 0x72, 0x64, 0x5C, 0x57, 0x32, 0x42,   //\wins.Record\W2B
    //  0x4E, 0x5C, 0x31, 0x5C, 0x6C, 0x6F, 0x73, 0x73, 0x65, 0x73, 0x00, 0x52, 0x65, 0x63, 0x6F, 0x72,   //N\1\losses.Recor
    //  0x64, 0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C, 0x31, 0x5C, 0x64, 0x69, 0x73, 0x63, 0x6F, 0x6E, 0x6E,   //d\W2BN\1\disconn
    //  0x65, 0x63, 0x74, 0x73, 0x00, 0x52, 0x65, 0x63, 0x6F, 0x72, 0x64, 0x5C, 0x57, 0x32, 0x42, 0x4E,   //ects.Record\W2BN
    //  0x5C, 0x31, 0x5C, 0x72, 0x61, 0x74, 0x69, 0x6E, 0x67, 0x00, 0x52, 0x65, 0x63, 0x6F, 0x72, 0x64,   //\1\rating.Record
    //  0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C, 0x31, 0x5C, 0x68, 0x69, 0x67, 0x68, 0x20, 0x72, 0x61, 0x74,   //\W2BN\1\high rat
    //  0x69, 0x6E, 0x67, 0x00, 0x44, 0x79, 0x6E, 0x4B, 0x65, 0x79, 0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C,   //ing.DynKey\W2BN\
    //  0x31, 0x5C, 0x72, 0x61, 0x6E, 0x6B, 0x00, 0x52, 0x65, 0x63, 0x6F, 0x72, 0x64, 0x5C, 0x57, 0x32,   //1\rank.Record\W2
    //  0x42, 0x4E, 0x5C, 0x31, 0x5C, 0x6C, 0x61, 0x73, 0x74, 0x20, 0x67, 0x61, 0x6D, 0x65, 0x00, 0x52,   //BN\1\last game.R
    //  0x65, 0x63, 0x6F, 0x72, 0x64, 0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C, 0x31, 0x5C, 0x6C, 0x61, 0x73,   //ecord\W2BN\1\las
    //  0x74, 0x20, 0x67, 0x61, 0x6D, 0x65, 0x20, 0x72, 0x65, 0x73, 0x75, 0x6C, 0x74, 0x00, 0x52, 0x65,   //t game result.Re
    //  0x63, 0x6F, 0x72, 0x64, 0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C, 0x33, 0x5C, 0x77, 0x69, 0x6E, 0x73,   //cord\W2BN\3\wins
    //  0x00, 0x52, 0x65, 0x63, 0x6F, 0x72, 0x64, 0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C, 0x33, 0x5C, 0x6C,   //.Record\W2BN\3\l
    //  0x6F, 0x73, 0x73, 0x65, 0x73, 0x00, 0x52, 0x65, 0x63, 0x6F, 0x72, 0x64, 0x5C, 0x57, 0x32, 0x42,   //osses.Record\W2B
    //  0x4E, 0x5C, 0x33, 0x5C, 0x64, 0x69, 0x73, 0x63, 0x6F, 0x6E, 0x6E, 0x65, 0x63, 0x74, 0x73, 0x00,   //N\3\disconnects.
    //  0x52, 0x65, 0x63, 0x6F, 0x72, 0x64, 0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C, 0x33, 0x5C, 0x72, 0x61,   //Record\W2BN\3\ra
    //  0x74, 0x69, 0x6E, 0x67, 0x00, 0x52, 0x65, 0x63, 0x6F, 0x72, 0x64, 0x5C, 0x57, 0x32, 0x42, 0x4E,   //ting.Record\W2BN
    //  0x5C, 0x33, 0x5C, 0x68, 0x69, 0x67, 0x68, 0x20, 0x72, 0x61, 0x74, 0x69, 0x6E, 0x67, 0x00, 0x44,   //\3\high rating.D
    //  0x79, 0x6E, 0x4B, 0x65, 0x79, 0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C, 0x33, 0x5C, 0x72, 0x61, 0x6E,   //ynKey\W2BN\3\ran
    //  0x6B, 0x00, 0x52, 0x65, 0x63, 0x6F, 0x72, 0x64, 0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C, 0x33, 0x5C,   //k.Record\W2BN\3\
    //  0x6C, 0x61, 0x73, 0x74, 0x20, 0x67, 0x61, 0x6D, 0x65, 0x00, 0x52, 0x65, 0x63, 0x6F, 0x72, 0x64,   //last game.Record
    //  0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C, 0x33, 0x5C, 0x6C, 0x61, 0x73, 0x74, 0x20, 0x67, 0x61, 0x6D,   //\W2BN\3\last gam
    //  0x65, 0x20, 0x72, 0x65, 0x73, 0x75, 0x6C, 0x74, 0x00, 0x00                     //e result..
    //};
   char fuckyou[] = {
     //0xFF, 0x26, 0x46, 0x02, 0x01, 0x00, 0x00, 0x00, 0x1A, 0x00, 0x00, 0x00, 0x55, 0x81, 0x73, 0x1A,  //  .&F.........U.s. < 0000   
     //0x61, 0x72, 0x69, 0x61, 0x39, 0x38, 0x31, 0x36, 0x00, 
     0x70, 0x72, 0x6F, 0x66, 0x69, 0x6C, 0x65,  //  aria9816.profile < 0010   
     0x5C, 0x73, 0x65, 0x78, 0x00, 0x70, 0x72, 0x6F, 0x66, 0x69, 0x6C, 0x65, 0x5C, 0x61, 0x67, 0x65,  //  \sex.profile\age < 0020   
     0x00, 0x70, 0x72, 0x6F, 0x66, 0x69, 0x6C, 0x65, 0x5C, 0x6C, 0x6F, 0x63, 0x61, 0x74, 0x69, 0x6F,  //  .profile\locatio < 0030   
     0x6E, 0x00, 0x70, 0x72, 0x6F, 0x66, 0x69, 0x6C, 0x65, 0x5C, 0x64, 0x65, 0x73, 0x63, 0x72, 0x69,  //  n.profile\descri < 0040   
     0x70, 0x74, 0x69, 0x6F, 0x6E, 0x00, 0x52, 0x65, 0x63, 0x6F, 0x72, 0x64, 0x5C, 0x57, 0x32, 0x42,  //  ption.Record\W2B < 0050   
     0x4E, 0x5C, 0x30, 0x5C, 0x77, 0x69, 0x6E, 0x73, 0x00, 0x52, 0x65, 0x63, 0x6F, 0x72, 0x64, 0x5C,  //  N\0\wins.Record\ < 0060   
     0x57, 0x32, 0x42, 0x4E, 0x5C, 0x30, 0x5C, 0x6C, 0x6F, 0x73, 0x73, 0x65, 0x73, 0x00, 0x52, 0x65,  //  W2BN\0\losses.Re < 0070   
     0x63, 0x6F, 0x72, 0x64, 0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C, 0x30, 0x5C, 0x64, 0x69, 0x73, 0x63,  //  cord\W2BN\0\disc < 0080   
     0x6F, 0x6E, 0x6E, 0x65, 0x63, 0x74, 0x73, 0x00, 0x52, 0x65, 0x63, 0x6F, 0x72, 0x64, 0x5C, 0x57,  //  onnects.Record\W < 0090   
     0x32, 0x42, 0x4E, 0x5C, 0x30, 0x5C, 0x6C, 0x61, 0x73, 0x74, 0x20, 0x67, 0x61, 0x6D, 0x65, 0x00,  //  2BN\0\last game. < 00A0   
     0x52, 0x65, 0x63, 0x6F, 0x72, 0x64, 0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C, 0x30, 0x5C, 0x6C, 0x61,  //  Record\W2BN\0\la < 00B0   
     0x73, 0x74, 0x20, 0x67, 0x61, 0x6D, 0x65, 0x20, 0x72, 0x65, 0x73, 0x75, 0x6C, 0x74, 0x00, 0x52,  //  st game result.R < 00C0   
     0x65, 0x63, 0x6F, 0x72, 0x64, 0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C, 0x31, 0x5C, 0x77, 0x69, 0x6E,  //  ecord\W2BN\1\win < 00D0   
     0x73, 0x00, 0x52, 0x65, 0x63, 0x6F, 0x72, 0x64, 0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C, 0x31, 0x5C,  //  s.Record\W2BN\1\ < 00E0   
     0x6C, 0x6F, 0x73, 0x73, 0x65, 0x73, 0x00, 0x52, 0x65, 0x63, 0x6F, 0x72, 0x64, 0x5C, 0x57, 0x32,  //  losses.Record\W2 < 00F0   
     0x42, 0x4E, 0x5C, 0x31, 0x5C, 0x64, 0x69, 0x73, 0x63, 0x6F, 0x6E, 0x6E, 0x65, 0x63, 0x74, 0x73,  //  BN\1\disconnects < 0100   
     0x00, 0x52, 0x65, 0x63, 0x6F, 0x72, 0x64, 0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C, 0x31, 0x5C, 0x72,  //  .Record\W2BN\1\r < 0110   
     0x61, 0x74, 0x69, 0x6E, 0x67, 0x00, 0x52, 0x65, 0x63, 0x6F, 0x72, 0x64, 0x5C, 0x57, 0x32, 0x42,  //  ating.Record\W2B < 0120   
     0x4E, 0x5C, 0x31, 0x5C, 0x68, 0x69, 0x67, 0x68, 0x20, 0x72, 0x61, 0x74, 0x69, 0x6E, 0x67, 0x00,  //  N\1\high rating. < 0130   
     0x44, 0x79, 0x6E, 0x4B, 0x65, 0x79, 0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C, 0x31, 0x5C, 0x72, 0x61,  //  DynKey\W2BN\1\ra < 0140   
     0x6E, 0x6B, 0x00, 0x52, 0x65, 0x63, 0x6F, 0x72, 0x64, 0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C, 0x31,  //  nk.Record\W2BN\1 < 0150   
     0x5C, 0x6C, 0x61, 0x73, 0x74, 0x20, 0x67, 0x61, 0x6D, 0x65, 0x00, 0x52, 0x65, 0x63, 0x6F, 0x72,  //  \last game.Recor < 0160   
     0x64, 0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C, 0x31, 0x5C, 0x6C, 0x61, 0x73, 0x74, 0x20, 0x67, 0x61,  //  d\W2BN\1\last ga < 0170   
     0x6D, 0x65, 0x20, 0x72, 0x65, 0x73, 0x75, 0x6C, 0x74, 0x00, 0x52, 0x65, 0x63, 0x6F, 0x72, 0x64,  //  me result.Record < 0180   
     0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C, 0x33, 0x5C, 0x77, 0x69, 0x6E, 0x73, 0x00, 0x52, 0x65, 0x63,  //  \W2BN\3\wins.Rec < 0190   
     0x6F, 0x72, 0x64, 0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C, 0x33, 0x5C, 0x6C, 0x6F, 0x73, 0x73, 0x65,  //  ord\W2BN\3\losse < 01A0   
     0x73, 0x00, 0x52, 0x65, 0x63, 0x6F, 0x72, 0x64, 0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C, 0x33, 0x5C,  //  s.Record\W2BN\3\ < 01B0   
     0x64, 0x69, 0x73, 0x63, 0x6F, 0x6E, 0x6E, 0x65, 0x63, 0x74, 0x73, 0x00, 0x52, 0x65, 0x63, 0x6F,  //  disconnects.Reco < 01C0   
     0x72, 0x64, 0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C, 0x33, 0x5C, 0x72, 0x61, 0x74, 0x69, 0x6E, 0x67,  //  rd\W2BN\3\rating < 01D0   
     0x00, 0x52, 0x65, 0x63, 0x6F, 0x72, 0x64, 0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C, 0x33, 0x5C, 0x68,  //  .Record\W2BN\3\h < 01E0   
     0x69, 0x67, 0x68, 0x20, 0x72, 0x61, 0x74, 0x69, 0x6E, 0x67, 0x00, 0x44, 0x79, 0x6E, 0x4B, 0x65,  //  igh rating.DynKe < 01F0   
     0x79, 0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C, 0x33, 0x5C, 0x72, 0x61, 0x6E, 0x6B, 0x00, 0x52, 0x65,  //  y\W2BN\3\rank.Re < 0200   
     0x63, 0x6F, 0x72, 0x64, 0x5C, 0x57, 0x32, 0x42, 0x4E, 0x5C, 0x33, 0x5C, 0x6C, 0x61, 0x73, 0x74,  //  cord\W2BN\3\last < 0210   
     0x20, 0x67, 0x61, 0x6D, 0x65, 0x00, 0x52, 0x65, 0x63, 0x6F, 0x72, 0x64, 0x5C, 0x57, 0x32, 0x42,  //   game.Record\W2B < 0220   
     0x4E, 0x5C, 0x33, 0x5C, 0x6C, 0x61, 0x73, 0x74, 0x20, 0x67, 0x61, 0x6D, 0x65, 0x20, 0x72, 0x65,  //  N\3\last game re < 0230   
     0x73, 0x75, 0x6C, 0x74, 0x00, 0x00                                                               //  sult..           < 0240   
  };

    // *(u16 *)(buf) = 0x26ff;
    // *(u16 *)(buf + 2) = 0x11 + strlen( name) + sizeof( fuckyou);
    // *(u32 *)(buf + 4) = 1;
    // *(u32 *)(buf + 8) = 1; //0x1A;
    // *(u32 *)(buf + 12) = rand(); //(u32)((time(0) - profile_lasttime) * 1000 + profile_lastval);
    // strcpy( buf + 0x10, name);
    // memcpy( buf + 0x11 + strlen( name), fuckyou, sizeof( fuckyou));    
    // disp->hexdump( *(u16 *)(buf + 2), buf);    
    // rawsend( *(u16 *)(buf + 2), buf);

  *this << (u32)1
       << (u32)6 // one object and name is included
       << (u32)rand()
       << name
       << "profile\\sex"
       << "profile\\age"
       << "profile\\location"
       << "profile\\description"
       << "profile\\publicrsaencryptkey"
       << "";
  sendpkt( 0x26ff);

  return( 0);
}


int Bot::setName( char *_username) {
  if (!_username) return( -1);
  strncpy( username, _username, sizeof( username) - 1);
  proxy->setName( username);
  return( 0);
}


int Bot::setPasswd( char *_password) {
  if (!_password) return( -1);
  strncpy( password, _password, sizeof( password) - 1);
  return( 0);
}


int Bot::setServer( char *_server) {
  if (!_server) return( -1);
  strncpy( server, _server, sizeof( server) - 1);
  proxy->setIP( server);
  return( 0);
}


int Bot::setHomeChannel( char *_homechannel) {
  if (!_homechannel) return( -1);
  strncpy( homechannel, _homechannel, sizeof( homechannel) -1);
  return( 0);
}


int Bot::setCDKey( char *_cdkey) {
  if (!_cdkey) return( -1);
  strncpy( cdkey, _cdkey, sizeof( cdkey) - 1);

  int len = 0; char *tmp = cdkey;
  while (*tmp) {
    if (*tmp != '-')
      len++;
    tmp++;
  }
  
  switch( len) {
    case 13: 
      if (clientType == -1)
        clientType = 1; 
      break;
    case 16: 
      if (clientType == -1)
        clientType = 0; 
      break;
    default:
      disp->clrline();
      *disp << CBRED << "Error: " << CBWHITE << "Your CD-Key has an invalid length." << endl;
      input->clear();
      return( -1);
  }

  return( 0);
}

int Bot::setCDKey2( char *_cdkey) {
  if (!_cdkey) return( -1);

  strncpy( cdkey2, _cdkey, sizeof( cdkey) - 1);
  return( 0);
}


int Bot::setNick( char *_nick) {
  if (!_nick) return( -1);

  strncpy( nick, _nick, sizeof( nick) - 1);
  return( 0);
}


int Bot::setRegister( char *_registered) {
  if (!_registered) return( -1);
  strncpy( registered, _registered, sizeof( registered) - 1);
  return( 0);
}


int Bot::setCommandSet( char *_cmdset) {
  if (!_cmdset) return( -1);
  strncpy( commandset, _cmdset, sizeof( commandset) - 1);
  return( 0);
}

int Bot::setSpawn( int _spawn) {
  spawn = _spawn ? 1 : 0;
  return( 0);
}

int Bot::setBind( char *_bind) {
  if (!_bind) return( -1);
  strncpy( bindaddy, _bind, sizeof( bindaddy) -1);
  return( 0);
}

int Bot::getDisplay( void) {
  return( display);
}

int Bot::setDisplay( char *_display) {
  if (!_display) return( -1);
  display = atoi( _display);
  if (display < 0) display = 0;
  if (display > 1) display = 1;

  disp->focus();

  return( 0);
}

int Bot::setSocks4_Server( char *_server) {
  if (!_server) return( -1);
  strncpy( socks4_server, _server, sizeof( socks4_server) - 1);
  return( 0);
}

int Bot::setSocks4_Port( char *_port) {
  if (!_port) return( -1);
  socks4_port = atoi( _port);
  return( 0);
}

int Bot::setSocks4_UserID( char *_userid) {
  if (!_userid) return( -1);
  strncpy( socks4_userid, _userid, sizeof( socks4_userid) - 1);
  return( 0);
}

char *Bot::getName( void) {
  return( username);
}


char *Bot::getPasswd( void) {
  return( password);
}


char *Bot::getServer( void) {
  return( server);
}


char *Bot::getHomeChannel( void) {
  return( homechannel);
}


char *Bot::getCDKey( void) {
  return( cdkey);
}

char *Bot::getCDKey2( void) {
  return( cdkey2);
}

char *Bot::getRegister( void) {
  return( registered);
}

char *Bot::getCommandSet( void) {
  return( commandset);
}

char *Bot::getCurChannel( void) {
  return( curchannel);
}

int Bot::getSpawn( void) {
  return( spawn);
}

int Bot::netclose( void) {
  disp->clrline();
  *disp << "Info:  " << getName() << " has closed connection." << endl;
  input->update();

  return( Network::netclose());
}

int Bot::enqueue( int lvl, char *str, ...) {
  va_list args; char buf[BUFSIZE];
  
  va_start( args, str);
  vsnprintf( buf, BUFSIZE, str, args);
  va_end( args);

  return( queue->enqueue( lvl, buf));
}

int Bot::setLagSpoof( char *_txt) {
  if (!_txt) return( -1);
  lag_spoof = atoi( _txt);
  return( 0);
}

CList *Bot::getCList( void) {
  return( clist);
}

int Bot::getClientType( void) {
  return( clientType);
}


int Bot::setIdle( int t, char *msg) {
  cmdIdle_d = t;

  if (!t) return( 0);

  #ifdef WIN32
    if (cmdIdle_d < 60) cmdIdle_d = 60;
  #else
    if (cmdIdle_d < 15)  cmdIdle_d = 15;
  #endif

  strncpy( cmdIdle_msg, msg, sizeof( cmdIdle_msg) - 1);

  return( 0);
}

int Bot::setMode( int _mode) {
  cmdMode = _mode;
}

int Bot::getMode( void) {
  return( cmdMode);
}

int Bot::setCommand( Command *_command) {
  command = _command;
  return( 0);
}

int Bot::focus( void) {
  disp->focus();
}

Proxy *Bot::getProxy( void) {
  return( proxy);
}

int Bot::getQuoteStatus( void) {
  return( quoteStatus);
}

void Bot::setQuoteStatus( int _quoteStatus) {
  quoteStatus = _quoteStatus;
}

void Bot::loadQuotes( char *_filename) {
  FILE *fd = fopen( _filename, "r");
  if (!fd) return;

  char buf[ 1024];
  while (!feof( fd)) {
    fgets( buf, sizeof( buf), fd);
    if (!feof( fd)) {
      if (char *t = strchr( buf, '\r')) *t = 0;
      if (char *t = strchr( buf, '\n')) *t = 0;

      // disp->clrline();
      // *disp << "Quote: " << buf << endl;
      // input->update();
      
      sQuotes *tmp = new sQuotes;
      tmp->data = new char[ strlen( buf) + 1];
      strcpy( tmp->data, buf);
      tmp->next = firstQuote;
      firstQuote = tmp;
      numQuotes++;
    }
  }
}

void Bot::checkQuotes( void) {
  if (!quoteStatus) return;
  if (!numQuotes) return;

  if ((time(0) - lastQuote) > quoteStatus) {
    int i = rand() % numQuotes;
    sQuotes *tmp = firstQuote;
    while (i--)
      tmp = tmp->next;
    enqueue( 7, "%s", tmp->data);
    lastQuote = time( 0);
  }
}

int Bot::setClient( char *_client) {
  if (!_client) return( -1);
  trim( _client);

  clientType = 1;  // Assume starcraft
  if (!strcasecmp( _client, "war2")) { clientType = 0; return( 0); }
  if (!strcasecmp( _client, "w2bn")) { clientType = 0; return( 0); }
  if (!strcasecmp( _client, "star")) { clientType = 1; return( 0); }
  if (!strcasecmp( _client, "jstr")) { clientType = 2; return( 0); }
  if (!strcasecmp( _client, "d2xp")) { clientType = 3; return( 0); }
  if (!strcasecmp( _client, "dexp")) { clientType = 3; return( 0); }

  return( -1);
}
