// Prolix - Sesquipedalian Chimerical Juju
//   Project distributed under the MIT License.
//
// Copyright (c) 2002 Eurijk!
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.

#ifndef OBJ_H 
#define OBJ_H

#include <stdio.h>

class Obj;

struct sLList {
	sLList *next;   
	Obj    *obj;
};
  
//===
// Comments appear before the object in
// the config files. All comments start
// with '#', and are automagically placed
// at the same indent level as the object.
struct sComments {
	sComments *next;
	char      *data;
};

class Obj {
private:
	char cwd[ 512];

	int _load( FILE *fd, int lvl);
	int _save( FILE *fd, int lvl);
	void _clear( void);
	void _init( void);

	void trim( char *buf);
	char *gettermn( char *buf, int n);
	char *gettermnz( char *buf, int n);
  
public:
	sComments  *commentFirst, *commentLast;
	
	char       name[ 33];
	
	// This should be protected or private, but we need other
	// classes to have easy access to this for now.
		int        dtype;         //  0 - List
		                          //  1 - int
		                          //  2 - char
		                          //  3 - data
		union {
			int      idata;
			char     *cdata;
			// sLList   *ldata;
		};

		sLList  	*objFirst,     *objLast;

	// Init
		Obj( void);
		Obj( const char *_name);
		Obj( const char *_name, const char *_cdata);
		~Obj( void);

	// Misc stack operations
		int append( Obj *obj);
		int push( Obj *obj);
		Obj *pop( void);

	// Recursive manipulators
		int save( const char *filename);
		int load( const char *filename);

	// This level data manipulators
		int commentAppend( const char *_comment);
    
		char *getName( void);
		void setName( const char *_name);
		void setData( const char *_cdata);
		void setData( int _idata);

	// Next level data manipulators
		char *getVar( const char *_name);
		int  getVarN( const char *_name);
		void setVar( const char *_name, const char *_cdata);
		void appendVar( const char *_name, const char *_cdata);
		void setVar( const char *_name, int _idata);
		Obj *getObj( const char *_name);
		void delObj( Obj *obj);
};

#endif
