#include <string.h>
#include "display.h"
#include "input.h"
#include "dyncmd.h"

extern Display *disp;
extern Input *input;

DynCmd::DynCmd( void) {
  first = keyfirst = 0;
}

DynCmd::~DynCmd( void) {
  clear();
}

void DynCmd::clear( void) {
  while (first) {
    sDynCmd *tmp = first->next;
    delete first->command;
    delete first->action;
    delete first;
    first = tmp;
  }
  while (keyfirst) {
    sDynCmd *tmp = keyfirst->next;
    delete keyfirst->command;
    delete keyfirst->action;
    delete keyfirst;
    keyfirst = tmp;
  }
}

int DynCmd::add( char *_command, char *_action) {
  sDynCmd *tmp = first;
  while (tmp) {
    if (!strcasecmp( tmp->command, _command))
      break;
    tmp = tmp->next;
  }

  if (!tmp) {
    tmp = new sDynCmd;
    tmp->command = new char[ strlen( _command) + 1];
    strcpy( tmp->command, _command);
    tmp->next = first;
    first = tmp;
  } else
    delete tmp->action;

  tmp->action = new char[ strlen( _action) + 1];
  strcpy( tmp->action, _action);

  return( 0);
}

int DynCmd::addkey( char *_command, char *_action) {
  sDynCmd *tmp = keyfirst;
  while (tmp) {
    if (!strcasecmp( tmp->command, _command))
      break;
    tmp = tmp->next;
  }

  if (!tmp) {
    tmp = new sDynCmd;
    tmp->command = new char[ strlen( _command) + 1];
    strcpy( tmp->command, _command);
    tmp->next = keyfirst;
    keyfirst = tmp;
  } else
    delete tmp->action;

  tmp->action = new char[ strlen( _action) + 1];
  strcpy( tmp->action, _action);

  return( 0);
}

int DynCmd::del( Bot *bot, char *_command) {
  sDynCmd **tmp = &first;
  while (*tmp) {
    if ( !strcasecmp((*tmp)->command, _command)) {
      sDynCmd *tmp2 = *tmp;
      *tmp = (*tmp)->next;

      delete tmp2->command;
      delete tmp2->action;
      delete tmp2;
      
      char buf[ 1024];
      sprintf( buf, "Command %s has been deleted", _command);
      bot->enqueue( 4, buf);
      return( 0);
    }
    tmp = &(*tmp)->next;
  }
   
  char buf[ 1024];
  sprintf( buf, "Command %s was not found.", _command);
  bot->enqueue( 4, buf);

  return( 0);
}

int DynCmd::delkey( Bot *bot, char *_command) {
  sDynCmd **tmp = &keyfirst;
  while (*tmp) {
    if ( !strcasecmp((*tmp)->command, _command)) {
      sDynCmd *tmp2 = *tmp;
      *tmp = (*tmp)->next;

      delete tmp2->command;
      delete tmp2->action;
      delete tmp2;
      
      char buf[ 1024];
      sprintf( buf, "Key %s has been deleted", _command);
      bot->enqueue( 4, buf);
      return( 0);
    }
    tmp = &(*tmp)->next;
  }
   
  char buf[ 1024];
  sprintf( buf, "Key %s was not found.", _command);
  bot->enqueue( 4, buf);

  return( 0);
}

void DynCmd::list( Bot *bot) {
  char buf[ 1024];

  sDynCmd *tmp = first;
  while (tmp) {
    sprintf( buf, "Cmd- %s %s", tmp->command, tmp->action);
    bot->enqueue( 4, buf);
    tmp = tmp->next;
  }

  tmp = keyfirst;
  while (tmp) {
    sprintf( buf, "Cmd+ %s %s", tmp->command, tmp->action);
    bot->enqueue( 4, buf);
    tmp = tmp->next;
  }
}

void DynCmd::handle( Bot *bot, char *_username, char *_command, char *_args) {
  #ifdef DEBUGDYNCMD
    disp->clrline();
    *disp << "DEBUGDYNCMD: " << _command << " | " << _args << endl;
    input->update();
  #endif

  sDynCmd *tmp = first;
  while (tmp) {
    if (!strcasecmp( _command, tmp->command)) {

      char outmsg[ 1024];
      char *outptr = outmsg;
      char *cmdptr = tmp->action;
      int  valid = 1;
      
      *outptr = 0;

      while (*cmdptr) {
        if (*cmdptr != '%')
          *outptr++ = *cmdptr++;
        else {
          cmdptr++;
          if ((*cmdptr >= '1') && (*cmdptr <= '9')) {
            char buf[ 1024];
            if (!_args)
              valid = 0;
            else {
              strcpy( buf, _args);
              char *tmp = gettermn( buf, *cmdptr - ('1' - 1));
              if (!tmp)
                valid = 0;
              else {
                while (*tmp)
                  *outptr++ = *tmp++;
                cmdptr++;
              }
            }
          } else if (*cmdptr == '0') {
            char *t = _username;
            while (*t)
              *outptr++ = *t++;
            cmdptr++;
          } else
            *outptr++ = *cmdptr++;
        }

      }
      
      *outptr = 0;
      if (valid) {
        bot->enqueue( 5, outmsg);
        return;
      }

    }
    tmp = tmp->next;
  }


  tmp = keyfirst;
  while (tmp) {
    int  valid = 0, i = 0;
    char buf[ 1024];
    if (!strcasecmp( _command, tmp->command)) valid++;

    if (_args) {
      char *tmp2;
      do {
        strcpy( buf, _args);
        if (tmp2 = gettermn( buf, i++))
          if (!strcasecmp( tmp->command, tmp2)) { 
            valid++;
            tmp2 = 0;
          }
      } while( tmp2);
    }

    if (valid) {
      char outmsg[ 1024];
      char *outptr = outmsg;
      char *cmdptr = tmp->action;
      
      *outptr = 0;

      while (*cmdptr) {
        if (*cmdptr != '%')
          *outptr++ = *cmdptr++;
        else {
          cmdptr++;
          if (*cmdptr == '0') {
            char *t = _username;
            while (*t)
              *outptr++ = *t++;
            cmdptr++;
          } else
            *outptr++ = *cmdptr++;
        }

      }
      
      *outptr = 0;
      bot->enqueue( 5, outmsg);
      return;

    }
    tmp = tmp->next;
  }

  
}
