// DesKey.cpp: implementation of the DesKey class.
//
//////////////////////////////////////////////////////////////////////

#include "des/des.h"
#include "deskey.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

DesKey::DesKey()
{

}

DesKey::~DesKey()
{

}

void DesKey::encrypt(const std::string &data, std::string &result) const
{
	unsigned char iv[8];
	unsigned char d[8];
	unsigned char *keys[3] = {(unsigned char*)key.c_str(), (unsigned char*)key.c_str()+7, (unsigned char*)key.c_str()+14};
	int truelen = data.length();
	std::string formatted;
	formatted.append((char*)&truelen, 4);
	formatted.append(data);
	formatted.resize((formatted.length()+7)/8*8, 0);
	unsigned char *plain = (unsigned char*)formatted.c_str();
	result.erase();
	memset(iv, 0, 8);
	for(int i = 0; i < formatted.length(); i += 8) {
		memcpy(d, plain + i, 8);
		for(int j = 0; j < 8; j++)
			d[j] ^= iv[j];
		des(d, keys[0], 1);
		des(d, keys[1], 0);
		des(d, keys[2], 1);
		memcpy(iv, d, 8);
		result.append((char*)d, 8);
	}
}

bool DesKey::decrypt(const std::string &data, std::string &result) const
{
	unsigned char iv[8];
	unsigned char d[8];
	unsigned char *keys[3] = {(unsigned char*)key.c_str(), (unsigned char*)key.c_str()+7, (unsigned char*)key.c_str()+14};
	unsigned char *cipher = (unsigned char*)data.c_str();
	int len = data.length();
	len = len / 8 * 8;
	if(len < 8)
		return false;
	result.erase();
	memset(iv, 0, 8);
	for(int i = 0; i < len; i += 8) {
		memcpy(d, cipher+i, 8);
		des(d, keys[2], 0);
		des(d, keys[1], 1);
		des(d, keys[0], 0);
		for(int j = 0; j < 8; j++)
			d[j] ^= iv[j];
		memcpy(iv, cipher+i, 8);
		result.append((char*)d, 8);
	}
	unsigned truelen = *(unsigned*)result.c_str();
	if(truelen > len-4) {
		return false;
	}
	result.erase(0, 4);
	result.resize(truelen);
	return true;
}

DesKey::DesKey(const DesKey & other) : description(other.description), key(other.key), source(other.source)
{
	calcid();
}

DesKey::DesKey(const std::string & k, const std::string &s, const std::string & d) : description(d), key(k), source(s)
{
	key.resize(21, 0);
	calcid();
}

unsigned DesKey::id() const
{
	return keyid;
}

void DesKey::calcid()
{
	unsigned nid[2] = {0,0};
	unsigned char *const data = (unsigned char*)nid;
	unsigned char *keys[3] = {(unsigned char*)key.c_str(), (unsigned char*)key.c_str()+7, (unsigned char*)key.c_str()+14};
	des(data, keys[0], 0);
	des(data, keys[1], 0);
	des(data, keys[2], 0);
	keyid = nid[0] ^ nid[1];
}

bool DesKey::operator ==(const DesKey & other) const
{
	return !key.compare(other.key);
}
