// Encryption.cpp: implementation of the Encryption class.
//
//////////////////////////////////////////////////////////////////////

#include "encryption.h"
#include <iostream>
#include <iomanip>

using namespace std;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Encryption::Encryption() {

}

Encryption::~Encryption() {

}

string Encryption::xbase64(const std::string &data) {
	unsigned i;
	string result;
	for(i = 0; i < data.length(); i += 3) {
		char c;
		result += char(0x80 + ((unsigned char)(data[i] & 0xfc) >> 2));
		c = char(0x80 + ((data[i] & 0x03) << 4));
		if(data.length() > i+1) {
			c += char((unsigned char)(data[i+1] & 0xf0) >> 4);
		}
		result += c;
		if(data.length() > i+1) {
			c = char(0x80 + ((unsigned char)(data[i+1] & 0x0f) << 2));
			if(data.length() > i+2) {
				c += char((unsigned char)(data[i+2] & 0xc0) >> 6);
			}
		} else {
			c = '=';
		}
		result += c;
		if(data.length() > i+2) {
			result += char(0x80 + (data[i+2] & 0x3f));
		} else {
			result += '=';
		}
	}
	return result;
}

string Encryption::dehex(const std::string &data) {
	if(data.length() % 2) {
		return "";
	}
	int i;
	string result;
	for(i = 0; i < data.length(); i += 2) {
		for(int j = 0; j < 2; j++) {
			if(data[i+j] < '0' || data[i+j] > '9' && data[i+j] < 'A' || data[i+j] > 'F' && data[i+j] < 'a' ||  data[i+j] > 'f')
				return "";
		}
		result += ((data[i] & 0x0f) << 4) + (data[i+1] & 0x0f) + (data[i] > '9'?0x90:0) + (data[i+1] > '9'?9:0);
	}
	return result;
}

string Encryption::xdebase64(const std::string &data) {
	if(data.length() % 4) {
		return "";
	}
	int i;
	string result;
	for(i = 0; i < data.length(); i += 4) {
		for(int j = 0; j < 3; j++) {
			if(((unsigned char)data[i+j] < 0x80 || (unsigned char)data[i+j] >= 0xc0) && data[i+j] != '=')
				return "";
		}
		result += ((data[i] & 0x3f) << 2) + ((data[i+1] & 0x30) >> 4);
		if(data[i+2] != '=') {
			result += ((data[i+1] & 0x0f) << 4) + ((data[i+2] & 0x3c) >> 2);
		}
		if(data[i+3] != '=') {
			result += ((data[i+2] & 0x03) << 6) + (data[i+3] & 0x3f);
		}
	}
	return result;
}


bool Encryption::importprivate(const std::string& rdata) {
	return importprivate(rdata, ownkey);
}

bool Encryption::importprivate(const std::string& rdata, private_key& key) {
	string data = rdata;
	if(data.length() < 5) {
		return false;
	}
	if(data[0] != 0x30) {
		return false;
	}
	data.erase(0, 1);
	int length = BERGetLength(data);
//	cout << "length " << length <<  " " << data.length() << endl;
	if(length-1 > data.length())
		return false;
	vlong n;
	if(!BERGetInteger(data, n)) // version
		return false;
	if(!BERGetInteger(data, key.m)) // n
		return false;
	if(!BERGetInteger(data, key.e)) // e
		return false;
	if(!BERGetInteger(data, n)) // d
		return false;
	if(!BERGetInteger(data, key.p)) // p
		return false;
	if(!BERGetInteger(data, key.q)) // q
		return false;
	if(!BERGetInteger(data, n)) // dp
		return false;
	if(!BERGetInteger(data, n)) // dq
		return false;
	if(!BERGetInteger(data, n)) // u
		return false;
	return true;
}

int Encryption::BERGetLength(std::string & data) {
	int length = 0;
	if(data.length() < 1) {
	} else if((unsigned char)data[0] <= 0x80) {
		length = (unsigned char)data[0];
		data.erase(0, 1);
	} else if(data.length() >= 1+ (unsigned char)data[0] - 0x80){
		int i;
		for(i = 1; i <= (unsigned char)data[0]-0x80; i++) {
			length <<= 8;
			length += (unsigned char)data[i];
		}
		data.erase(0, 1+(unsigned char)data[0]-0x80);
	}
	return length;
}




bool Encryption::BERGetInteger(std::string & data, vlong& n)
{
	if(data.length() < 3)
		return false;
	if(data[0] != 2)
		return false;
	data.erase(0, 1);
	int length = BERGetLength(data);
	if(length > data.length())
		return false;
	n.load(data.c_str(), length);

//	cout << setbase(16);
//	for(int i = 0; i < length; i++) {
//		cout << setw(2) << setfill('0') << (int)(unsigned char)data[i] << " ";
//	}
//	cout << setbase(10);
//	cout << endl;
	data.erase(0, length);
	return true;
}

int Encryption::process(const std::string &encrypted, const std::string &user, std::string &result, PubKey *&signer) {
	int resultcode = 0;
	signer = 0;
	string binary = xdebase64(encrypted);
	if(binary.length() < 1) {
		return BADLENGTH|resultcode;
	}
	int flags = binary[0];
	if((flags & 0xe0) != CRYPTOVERSION)
		return BADVERSION|resultcode;
	while(flags & 3) {
		switch(flags & 3) {
		case 1: // Conventional encryption
			{
			resultcode |= DES;
			if(binary.length() < 1 + 4)
				return BADLENGTH|resultcode;
			unsigned keyid;
			binary.copy((char*)&keyid, 4, 1);
			DesKeys::iterator it = deskeys.find(keyid);
			if(it == deskeys.end())
				return NOKEY|resultcode;
			while(it->first == keyid) {
				DesKey& key = it->second;
				string decrypted;
				if(key.decrypt(binary.substr(5), decrypted)) {
					binary = decrypted;
					break;
				}
				it++;
			}
			if(it->first != keyid)
				return NOKEY|resultcode;
			}
			break;
		case 2: // Public key encryption
			{
			resultcode |= RSA;
			if(binary.length() != 129)
				return BADLENGTH|resultcode;
			if(!PubKey::decrypt(binary.substr(1), binary, ownkey))
				return NOKEY|resultcode;
			}
			break;
		case 3:
			return UNSUPPORTED|resultcode;
		}
		if(binary.length() < 1)
			return BADLENGTH|resultcode;
		flags = binary[0];
		if((flags & 0xe0) != 0xe0)
			return BADVERSION|resultcode;
	}
	switch(flags & 0xc) {
	case 0x04:// New des key
		{
		resultcode |= KEY;
		if(binary.length() < 22)
			return BADLENGTH|resultcode;
		binary.erase(0, 1);
		DesKey key(binary, user, binary.c_str() + 21);
		DesKeys::iterator it = deskeys.find(key.id());
		while(it != deskeys.end() && it->first == key.id()) {
//			if(it->second == key)
//				it = deskeys.erase(it);
//			else
//				it++;
		}
		deskeys.insert(DesKeys::value_type(key.id(), key));
		result = binary.substr(21);
		}
		break;
	case 0x00:// Text message
		{
		resultcode |= TEXT;
		result = binary.substr(1);
		}
		break;
	case 0x08:
	case 0x0c:
		return UNSUPPORTED|resultcode;
	}
//	unsigned signature[5];
//	if(flags & 4) {
//		resultcode |= SIGNED;
//		if(binary.length() < 1 + 4 + 128)
//			return CRYPTOERROR;
//		unsigned signerid;
//		binary.copy((char*)&signerid, 4, 1);
//		KeyRing::iterator it = keyring.find(signerid);
//		if(it == keyring.end())
//			return CRYPTOERROR|SIGNED;
//		signer = &it->second;
//	}
	return resultcode;
}
	


std::string Encryption::encrypttext(const std::string &txt, const DesKey & target) {
	std::string result;
	std::string temp(1, char(CRYPTOVERSION|0x00));
	temp += txt;
	target.encrypt(temp, result);
	char header[5];
	header[0] = char(CRYPTOVERSION|0x01);
	*(unsigned*)(header+1) = target.id();
	return xbase64(std::string(header, 5) + result);
}

std::string Encryption::encryptkey(const DesKey & key, const PubKey & target) {
	std::string data(1, char(CRYPTOVERSION|0x04));
	data += key.key + key.description;
	if(data.size() > 123)
		data.resize(123);
	std::string encrypted;
	encrypted.reserve(129);
	if(!target.encrypt(data, encrypted))
		return "";
	encrypted.insert(std::string::size_type(0), std::string::size_type(1), char(CRYPTOVERSION|0x02));
	return xbase64(encrypted);
}

bool Encryption::importpublic(const std::string &rdata, PubKey &key) {
	return importpublic(rdata, key.key);
}

bool Encryption::importpublic(const std::string & rdata, public_key & key) {
	string data = rdata;
	if(data.length() < 5) {
		return false;
	}
	if(data[0] != 0x30) {
		return false;
	}
	data.erase(0, 1);
	int length = BERGetLength(data);
//	cout << "length " << length <<  " " << data.length() << endl;
	if(length-1 > data.length())
		return false;
	vlong n;
	if(!BERGetInteger(data, key.m)) // n
		return false;
	if(!BERGetInteger(data, key.e)) // e
		return false;
	return true;
}

std::string Encryption::encrypttext(const std::string &txt, const PubKey & target) {
	std::string data(1, char(CRYPTOVERSION|0x00));
	data += txt;
	if(data.size() > 123)
		data.resize(123);
	std::string encrypted;
	encrypted.reserve(129);
	if(!target.encrypt(data, encrypted))
		return "";
	encrypted.insert(std::string::size_type(0), std::string::size_type(1), char(CRYPTOVERSION|0x02));
	return xbase64(encrypted);
}

std::string Encryption::hex(const std::string &data) {
	std::string result;
	result.reserve(data.length() * 2);
	static const char table[] = {"0123456789ABCDEF"};
	for(int i = 0; i < data.length(); i++) {
		result += table[(data[i] >> 4) & 0x0f];
		result += table[data[i] & 0x0f];
	}
	return result;
}
