#include <stdarg.h>
#include <string.h>
#include "globals.h"
#include "enqueue.h"

Enqueue::Enqueue( void) {
	pthread_mutex_init( &enqueue_mutex, 0);

	handlers = 0;

	for (int i = 0; i < (ENQUEUE_LEVELS - 1); i++) {
		first[ i] = last[ i] = 0;
		depth[ i] = 0;
	}
}

Enqueue::~Enqueue( void) {
  while (handlers) {
    sHandlers *tmp = handlers->next;
    delete handlers;
    handlers = tmp;
  }  

  delqueue();
}


int Enqueue::delqueue( void) {
	pthread_mutex_lock( &enqueue_mutex);
	
	for (int i = 0; i < (ENQUEUE_LEVELS - 1); i++) {
		while (first[ i]) {
			struct sQueue *tmp = first[i]->next;
			delete first[i]->buf;
			delete first[i];
			first[i] = tmp;
		}
		first[i] = last[i] = 0;
		depth[i] = 0;
	}

	pthread_mutex_unlock( &enqueue_mutex);
}


int Enqueue::deQueue( struct sMessage *smsg) {
	if (!smsg) return( -1);

	pthread_mutex_lock( &enqueue_mutex);

	for (int i = 0; i < (ENQUEUE_LEVELS - 1); i++) {
		if (first[ i]) {
			smsg->lvl = i;
			smsg->buf = first[i]->buf;
      
      struct sQueue *tmp = first[i]->next;
      delete first[i];
      first[i] = tmp;
      if (!first[i])
        last[i] = 0;
      depth[i]--;
		pthread_mutex_unlock( &enqueue_mutex);
      return( 0);
    }
  }
	pthread_mutex_unlock( &enqueue_mutex);
  return( -1);
}

int Enqueue::enQueue( struct sMessage *smsg) {
	if (!smsg) return( -1);

	pthread_mutex_lock( &enqueue_mutex);

  try { 
    struct sQueue *tmp = new sQueue;
    tmp->next = first[ smsg->lvl];
    tmp->buf = smsg->buf;

    first[ smsg->lvl] = tmp;
    depth[ smsg->lvl]++;

    if (!last[ smsg->lvl])
      last[ smsg->lvl] = tmp;
	}
	catch( ... ) {
		pthread_mutex_unlock( &enqueue_mutex);
		return( -1);
	}

	pthread_mutex_unlock( &enqueue_mutex);
	return( 0);
}

int Enqueue::addHandler( Bot *bot) {
	pthread_mutex_lock( &enqueue_mutex);
	
	try {
		struct sHandlers *tmp = new sHandlers;
    tmp->next = handlers;
    tmp->bot = bot;
    handlers = tmp;
	}
	catch( ... ) {
		pthread_mutex_unlock( &enqueue_mutex);
    return( -1);
  }

	pthread_mutex_unlock( &enqueue_mutex);
  return( 0);
}

int Enqueue::deleteHandler( Bot *bot) {
	pthread_mutex_lock( &enqueue_mutex);

  struct sHandlers *tmp = handlers;
  if (!tmp) {
  	pthread_mutex_unlock( &enqueue_mutex);

  	return( -1);
  }

  if (tmp->bot == bot) {
    tmp = handlers->next;
    delete handlers;
    handlers = tmp;
	pthread_mutex_unlock( &enqueue_mutex);
    return( 0);
  }
  
  while (tmp->next) {
    if (tmp->next->bot == bot) {
      sHandlers *tmp2 = tmp->next;
      tmp->next = tmp2->next;
      delete tmp2;
	pthread_mutex_unlock( &enqueue_mutex);
      return( 0);
    }
    tmp = tmp->next;
  }
	pthread_mutex_unlock( &enqueue_mutex);
  return( -1);
}

int Enqueue::enqueue( int lvl, char *str, ...) {
	va_list args; char buf[BUFSIZE];
  
	va_start( args, str);
	#ifdef WIN32
		vsprintf( buf, str, args);
	#else
		vsnprintf( buf, BUFSIZE, str, args);
	#endif
	va_end( args);
	buf[ sizeof( buf) - 1] = '\0';

	try {
		if (lvl == (ENQUEUE_LEVELS - 1)) {
			for (int i = 0; i < (ENQUEUE_LEVELS - 1); i++)
				if (depth[ i]) return( -1);
			return( enqueue( 3, buf));
		}

		if (depth[ lvl] >= ENQUEUE_DEPTH)
			return( -1);

		if (last[ lvl]) {
			last[ lvl]->next = new sQueue;
			last[ lvl] = last[ lvl]->next;
		} else
			first[ lvl] = last[ lvl] = new sQueue;

		last[ lvl]->next = 0;
		depth[ lvl]++;

		last[ lvl]->buf = new char[ strlen( buf) + 1];
		strcpy( last[ lvl]->buf, buf);

	}
	catch ( ... ) {
		return( -1);
	}

	//===
	// Signal the main thread.
		globals->tio->signal();

	return( idle());
}

int Enqueue::idle( void) {
  sMessage smsg;

  for (;;) {
    smsg.buf = 0;

    if (deQueue( &smsg))
      break;

    int ret = 0;
    sHandlers *shand = handlers;
    while (shand) {
      if (!shand->bot->pkt_send( strlen( smsg.buf), smsg.buf)) {
        delete smsg.buf;
        smsg.buf = 0;
        break;
      }
      shand = shand->next;
    }

    if (!shand)
      break;
  }

  if (smsg.buf)
    enQueue( &smsg);

  return( 0);
}
