#include "globals.h"

#include "botlist.h"
#include "enqueue.h"
#include "commandlist.h"
#include "command.h"

extern CommandList commandlist;
extern BotList botlist;
extern Enqueue enqueue;

Globals::Globals( void) {
	debug(("Globals::Globals() - start"));

	done = 0;

	db = new Obj();
	db->load("nconfig.txt");

	userdb = new Obj();
	userdb->load("userdb.txt");

	debug(("Globals::Globals() - done loading nconfig and userdb .txt"));

	//===
	// Make sure we have a minimal config, todo, cleanup :)
	{
		Obj *obj = db->getObj("vars");
		if (!obj) {
			obj = new Obj("vars");
			db->append( obj);
		}
	}


	//===
	// Load all the command sets
		{
			Obj *objCmd = db->getObj("dyncmd");
			if (!objCmd) {
				objCmd = new Obj("dyncmd");
				db->append( objCmd);
			}
		
			debug(("Globals - traversing objBots"));
			sLList *tmp = objCmd->objFirst;
			while (tmp) {
				if (tmp->obj) {
					Command *command = new Command();
					command->setName( tmp->obj->getName());
					commandlist.add( command);
				}
				tmp = tmp->next;
			}
		}
		
	//===
	// Load all the bots.
		Obj *objBots = db->getObj("bots");
		if (!objBots) {
			objBots = new Obj("bots");
			db->append( objBots);
		}

		debug(("Globals - traversing objBots"));
		sLList *tmp = objBots->objFirst;
		while (tmp) {
			if (tmp->obj) {
				char *username = tmp->obj->getName();
				if (username && *username) {
					debug(("Globals() - adding a bot"));
					Bot *bot = new Bot();
					bot->setObj( tmp->obj);
				
					char *passwd = tmp->obj->getVar("passwd");
					char *server = tmp->obj->getVar("server");
					char *regto = tmp->obj->getVar("regto");
					char *cdkey = tmp->obj->getVar("cdkey");
					char *cdkey2 = tmp->obj->getVar("cdkey2");
					char *pserver = tmp->obj->getVar("pserver");
					char *pport = tmp->obj->getVar("pport");
					char *puser = tmp->obj->getVar("puser");
					char *client = tmp->obj->getVar("client");
					char *homechannel = tmp->obj->getVar("homechannel");
					char *display = tmp->obj->getVar("display");
					char *spawn = tmp->obj->getVar("spawn");
					char *commandset = tmp->obj->getVar("commandset");
					char *active = tmp->obj->getVar("active");
					char *plug = tmp->obj->getVar("plug");

					bot->setName( username);
					bot->setPasswd( passwd);
					bot->setServer( server);
					bot->setCDKey( cdkey);
					bot->setCDKey2( cdkey2);
					bot->setNick("");
					bot->setClient( client);
					bot->setHomeChannel( homechannel);
					bot->setRegister( regto);

					bot->setBind( tmp->obj->getVar("bind"));

					bot->setSocks4_Server( pserver);
					bot->setSocks4_Port( pport);
					bot->setSocks4_UserID( puser);

					if (commandset) bot->setCommandSet( commandset); else bot->setCommandSet( "default");
					if (spawn)      bot->setSpawn( atoi( spawn));    else bot->setSpawn( 0);
					if (active)     bot->setActive( atoi( active));
					if (plug)       bot->setPlug( atoi( plug));

					enqueue.addHandler( bot);
					botlist.add( bot);
					debug(("Globals() - done adding a bot"));
				}
			}
			tmp = tmp->next;
		}
		
	#ifdef DISP_GTK
		tgtk = NULL;
	#endif
	tio = NULL;

	debug(("Globals::Globals() - done"));
}

Globals::~Globals( void) {
	debug(("Globals::~Globals() - start"));

	db->save("nconfig.txt");
	delete db;
	
	userdb->save("userdb.txt");
	delete userdb;

	debug(("Globals::~Globals() - done"));
}
