#include <stdio.h>
#include "../globals.h"
#include "../tools/build.h"
#include "wndnewbot.h"
#include "wndalert.h"
#include "../enqueue.h"
#include "../botlist.h"

extern char *WndHash_clients[];
extern Enqueue enqueue;
extern BotList botlist;

void onWndNewBot_unrealize( GtkWidget *widget, gpointer user_data) {
}


void onWndNewBot_button1_clicked( GtkButton *button, gpointer user_data) {
	((WndNewBot *)user_data)->newBot();
}


void onWndNewBot_button2_clicked            (GtkButton       *button,
                                        gpointer         user_data) {

	gtk_widget_hide( ((WndNewBot *)user_data)->wndnewbot);
}

void WndNewBot::newBot( void) {
	Obj *obj = NULL;

	try {
		//===
		// Get the name
			const char *username = gtk_entry_get_text( (GtkEntry *)entry1);
			if (!username || !*username)
				throw("Missing username. You must have a username for the bot");

		//===
		// Create out new bot object.
			if (bot)
				obj = bot->objbot;
			else
				obj = new Obj( username);

		//===
		// Get the password
			const char *passwd =  gtk_entry_get_text( (GtkEntry *)entry2);
			if (!passwd || !*passwd)
				throw("Missing password. You must have a password for the bot");

		//===
		// Make sure we have a server.
			const char *server = gtk_entry_get_text( (GtkEntry *)entry3);
			if (!server || !*server)
				throw("Missing server. You must have a server to connect to.");
		
		//===
		// Grab some other settings.
			debug(("WndNewBot - grabbing other settings"));
			const char *display     = "1";
			const char *regto       = gtk_entry_get_text( (GtkEntry *)entry4);
			const char *cdkey       = gtk_entry_get_text( (GtkEntry *)entry5);
			const char *cdkey2      = gtk_entry_get_text( (GtkEntry *)entry6);
			const char *pserver     = gtk_entry_get_text( (GtkEntry *)entry7);
			const char *pport       = gtk_entry_get_text( (GtkEntry *)entry8);
			const char *puser       = gtk_entry_get_text( (GtkEntry *)entry9);
			const char *client      = "W2BN";
			const char *homechannel = gtk_entry_get_text( (GtkEntry *)entry15);
			int  myactive     = 0;

			{
				GtkWidget *active = gtk_menu_get_active( (GtkMenu *)optionmenu5_menu);
				for (int i = 0; i < WNDHASH_CLIENTS; i++) {
					if ( glade_menuitem[i] == (GtkWidget *)active) {
						client = WndHash_clients[i];
					}
				}
			}

			{
				GtkWidget *active = gtk_menu_get_active( (GtkMenu *)optionmenu3_menu);
				for (int i = 0; i < 2; i++)
					if ( glade_menuitem2[i] == (GtkWidget *)active)
						myactive = i;
			}

		//===
		// Set our vars.
			debug(("WndNewBot - setting our vars"));
			if (!passwd)  passwd = "";
			if (!server)  server = "";
			if (!regto)   regto = "";
			if (!cdkey)   cdkey = "";
			if (!cdkey2)  cdkey2 = "";
			if (!pserver) pserver = "";
			if (!pport)   pport = "";
			if (!puser)   puser = "";
			if (!homechannel) homechannel = "clan [ds]";
		
			obj->setVar( "display",     display);
			obj->setVar( "passwd",      passwd);
			obj->setVar( "server",      server);
			obj->setVar( "register",    regto);
			obj->setVar( "cdkey",       cdkey);
			obj->setVar( "cdkey2",      cdkey2);
			obj->setVar( "pserver",     pserver);
			obj->setVar( "pport",       pport);
			obj->setVar( "puser",       puser);
			obj->setVar( "client",      client);
			obj->setVar( "homechannel", homechannel);
			if (myactive)
				obj->setVar( "active",   "1");
			else
				obj->setVar( "active",   "0");
			// obj->setVar( "commandset",  "Default");

			{
				GtkWidget *active = gtk_menu_get_active( (GtkMenu *)optionmenu6_menu);
				obj->setVar( "commandset", (*mapCmdSet)[active].c_str());
			}
			
			if (bot) {
				bot->setServer( server);
			}

		//===
		// If we are just updating, then stop here
			if (bot) {
				gtk_widget_hide( wndnewbot);
				return;
			}
		
		//===
		// Add the bot to the global bot list.
			debug(("WndNewBot - adding bot db object"));
			Obj *bots = globals->db->getObj("bots");
			if (!bots) {
				bots = new Obj("bots");
				globals->db->append( bots);
			}
			
			bots->append( obj);

		//===
		// Todo: Actually create the bot.
			debug(("WndNewBot - actually creating the bot"));
			bot = new Bot();
			debug(("WndNewBot - actually creating the bot - 0"));
			bot->setName( username);
			debug(("WndNewBot - actually creating the bot - 0.5"));
			bot->setPasswd( passwd);
			bot->setServer( server);
			debug(("WndNewBot - actually creating the bot - 1"));
			bot->setCDKey( cdkey);
			bot->setCDKey2( cdkey2);
			debug(("WndNewBot - actually creating the bot - 2"));
			bot->setNick("");			
			debug(("WndNewBot - actually creating the bot - 2.2"));
			bot->setClient( client);
			debug(("WndNewBot - actually creating the bot - 2.4"));
			bot->setHomeChannel( homechannel);
			debug(("WndNewBot - actually creating the bot - 3"));
			bot->setRegister( regto);
			bot->setCommandSet( obj->getVar("commandset"));
			bot->setSpawn( 0);
			// bot->setBind("");
			// bot->setDisplay();
			// bot->setLagSpoof();

		//===
		// Append the bot to the botlist
			debug(("WndNewBot - appending the bot"));
			enqueue.addHandler( bot);
			botlist.add( bot);
		
		//===
		// All done, destory ourself
			debug(("WndNewBot - hiding ourself"));
			gtk_widget_hide( wndnewbot);

	} catch( const char *msg) {
		new WndAlert( msg);
		if (obj) {
			delete obj;
			obj = NULL;
		}
	}
}

WndNewBot::WndNewBot( Bot *bot) {
	WndNewBot::bot = bot;
	mapCmdSet = new std::map<GtkWidget *,std::string>;

	wndnewbot = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_object_set_data (GTK_OBJECT (wndnewbot), "WndNewBot", wndnewbot);
	gtk_container_set_border_width (GTK_CONTAINER (wndnewbot), 2);
	
	{
		char buf[ 256];
		if (bot)
			sprintf( buf, "zDSBot3 (build %d) - Edit Bot", BUILD);
		else
			sprintf( buf, "zDSBot3 (build %d) - New Bot", BUILD);
		gtk_window_set_title (GTK_WINDOW (wndnewbot), buf);
	}

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (vbox1);
	gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "vbox1", vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (vbox1);
	gtk_container_add (GTK_CONTAINER (wndnewbot), vbox1);

  frame2 = gtk_frame_new ("General");
  gtk_widget_ref (frame2);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "frame2", frame2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame2);
  gtk_box_pack_start (GTK_BOX (vbox1), frame2, TRUE, TRUE, 0);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox2);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "vbox2", vbox2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox2);
  gtk_container_add (GTK_CONTAINER (frame2), vbox2);

  table2 = gtk_table_new (5, 2, FALSE);
  gtk_widget_ref (table2);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "table2", table2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table2);
  gtk_box_pack_start (GTK_BOX (vbox2), table2, FALSE, TRUE, 0);

  label1 = gtk_label_new ("Username");
  gtk_widget_ref (label1);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "label1", label1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label1);
  gtk_table_attach (GTK_TABLE (table2), label1, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (label1, 80, -2);
  gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (label1), 2, 2);

  label2 = gtk_label_new ("Password");
  gtk_widget_ref (label2);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "label2", label2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label2);
  gtk_table_attach (GTK_TABLE (table2), label2, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (label2), 2, 2);

  label3 = gtk_label_new ("Server");
  gtk_widget_ref (label3);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "label3", label3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label3);
  gtk_table_attach (GTK_TABLE (table2), label3, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label3), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (label3), 2, 2);

  label4 = gtk_label_new ("Register");
  gtk_widget_ref (label4);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "label4", label4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label4);
  gtk_table_attach (GTK_TABLE (table2), label4, 0, 1, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label4), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (label4), 2, 2);

  entry1 = gtk_entry_new ();
  gtk_widget_ref (entry1);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "entry1", entry1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (entry1);
  gtk_table_attach (GTK_TABLE (table2), entry1, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 2, 1);

  entry2 = gtk_entry_new ();
  gtk_widget_ref (entry2);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "entry2", entry2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (entry2);
  gtk_table_attach (GTK_TABLE (table2), entry2, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 2, 1);

	gtk_entry_set_visibility((GtkEntry *)entry2, FALSE);

  entry3 = gtk_entry_new ();
  gtk_widget_ref (entry3);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "entry3", entry3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (entry3);
  gtk_table_attach (GTK_TABLE (table2), entry3, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 2, 1);

  entry4 = gtk_entry_new ();
  gtk_widget_ref (entry4);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "entry4", entry4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (entry4);
  gtk_table_attach (GTK_TABLE (table2), entry4, 1, 2, 3, 4,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 2, 1);

  label27 = gtk_label_new ("Homechannel");
  gtk_widget_ref (label27);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "label27", label27,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label27);
  gtk_table_attach (GTK_TABLE (table2), label27, 0, 1, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 2, 2);
  gtk_misc_set_alignment (GTK_MISC (label27), 0, 0.5);

  entry15 = gtk_entry_new ();
  gtk_widget_ref (entry15);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "entry15", entry15,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (entry15);
  gtk_table_attach (GTK_TABLE (table2), entry15, 1, 2, 4, 5,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 2, 2);


  hseparator1 = gtk_hseparator_new ();
  gtk_widget_ref (hseparator1);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "hseparator1", hseparator1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hseparator1);
  gtk_box_pack_start (GTK_BOX (vbox2), hseparator1, TRUE, TRUE, 2);

  table4 = gtk_table_new (6, 2, FALSE);
  gtk_widget_ref (table4);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "table4", table4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table4);
  gtk_box_pack_start (GTK_BOX (vbox2), table4, FALSE, TRUE, 0);

  label5 = gtk_label_new ("Client");
  gtk_widget_ref (label5);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "label5", label5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label5);
  gtk_table_attach (GTK_TABLE (table4), label5, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (label5, 80, -2);
  gtk_misc_set_alignment (GTK_MISC (label5), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (label5), 2, 2);

  label6 = gtk_label_new ("Spawn");
  gtk_widget_ref (label6);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "label6", label6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label6);
  gtk_table_attach (GTK_TABLE (table4), label6, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label6), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (label6), 2, 2);

  label7 = gtk_label_new ("CD-Key");
  gtk_widget_ref (label7);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "label7", label7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label7);
  gtk_table_attach (GTK_TABLE (table4), label7, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label7), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (label7), 2, 2);

  label8 = gtk_label_new ("Exp CD-Key");
  gtk_widget_ref (label8);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "label8", label8,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label8);
  gtk_table_attach (GTK_TABLE (table4), label8, 0, 1, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label8), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (label8), 2, 2);

  label9 = gtk_label_new ("Active");
  gtk_widget_ref (label9);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "label9", label9,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label9);
  gtk_table_attach (GTK_TABLE (table4), label9, 0, 1, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label9), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (label9), 2, 2);

  entry5 = gtk_entry_new ();
  gtk_widget_ref (entry5);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "entry5", entry5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (entry5);
  gtk_table_attach (GTK_TABLE (table4), entry5, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 2, 1);

  entry6 = gtk_entry_new ();
  gtk_widget_ref (entry6);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "entry6", entry6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (entry6);
  gtk_table_attach (GTK_TABLE (table4), entry6, 1, 2, 3, 4,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 2, 1);

  optionmenu3 = gtk_option_menu_new ();
  gtk_widget_ref (optionmenu3);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "optionmenu3", optionmenu3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (optionmenu3);
  gtk_table_attach (GTK_TABLE (table4), optionmenu3, 1, 2, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 2, 1);
  optionmenu3_menu = gtk_menu_new ();
	{
		glade_menuitem2[0] = gtk_menu_item_new_with_label ("Disabled on Start");
		gtk_widget_show (glade_menuitem2[0]);
		gtk_menu_append (GTK_MENU (optionmenu3_menu), glade_menuitem2[0]);

		glade_menuitem2[1] = gtk_menu_item_new_with_label ("Enabled on Start");
		gtk_widget_show (glade_menuitem2[1]);
		gtk_menu_append (GTK_MENU (optionmenu3_menu), glade_menuitem2[1]);

		gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu3), optionmenu3_menu);
	}

  optionmenu4 = gtk_option_menu_new ();
  gtk_widget_ref (optionmenu4);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "optionmenu4", optionmenu4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (optionmenu4);
  gtk_table_attach (GTK_TABLE (table4), optionmenu4, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 2, 1);
  optionmenu4_menu = gtk_menu_new ();
	{ // Spawn settings
		GtkWidget *glade_menuitem = gtk_menu_item_new_with_label ("No Spawn");
		gtk_widget_show (glade_menuitem);
		gtk_menu_append (GTK_MENU (optionmenu4_menu), glade_menuitem);
		glade_menuitem = gtk_menu_item_new_with_label ("Yes Spawn");
		gtk_widget_show (glade_menuitem);
		gtk_menu_append (GTK_MENU (optionmenu4_menu), glade_menuitem);
		gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu4), optionmenu4_menu);
	}

  optionmenu5 = gtk_option_menu_new ();
  gtk_widget_ref (optionmenu5);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "optionmenu5", optionmenu5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (optionmenu5);
  gtk_table_attach (GTK_TABLE (table4), optionmenu5, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 2, 1);
  optionmenu5_menu = gtk_menu_new ();

	//===
	// Add the pull-down client select.
		for (int i = 0; i < WNDHASH_CLIENTS; i++) {
			glade_menuitem[i] = gtk_menu_item_new_with_label (WndHash_clients[i]);
			gtk_widget_show (glade_menuitem[i]);
			gtk_menu_append (GTK_MENU (optionmenu5_menu), glade_menuitem[i]);
		}

  //glade_menuitem = gtk_menu_item_new_with_label ("W2BN");
  //gtk_widget_show (glade_menuitem);
  //gtk_menu_append (GTK_MENU (optionmenu5_menu), glade_menuitem);
  //glade_menuitem = gtk_menu_item_new_with_label ("STAR");
  //gtk_widget_show (glade_menuitem);
  //gtk_menu_append (GTK_MENU (optionmenu5_menu), glade_menuitem);
  //glade_menuitem = gtk_menu_item_new_with_label ("SEXP");
  //gtk_widget_show (glade_menuitem);
  //gtk_menu_append (GTK_MENU (optionmenu5_menu), glade_menuitem);
  //glade_menuitem = gtk_menu_item_new_with_label ("JSTR");
  //gtk_widget_show (glade_menuitem);
  //gtk_menu_append (GTK_MENU (optionmenu5_menu), glade_menuitem);

  gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu5), optionmenu5_menu);

  label15 = gtk_label_new ("CommandSet");
  gtk_widget_ref (label15);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "label15", label15,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label15);
  gtk_table_attach (GTK_TABLE (table4), label15, 0, 1, 5, 6,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label15), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (label15), 2, 2);

  optionmenu6 = gtk_option_menu_new ();
  gtk_widget_ref (optionmenu6);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "optionmenu6", optionmenu6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (optionmenu6);
  gtk_table_attach (GTK_TABLE (table4), optionmenu6, 1, 2, 5, 6,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 2, 1);
  optionmenu6_menu = gtk_menu_new ();

	{
		Obj *commandsets = globals->db->getObj("dyncmd");
		if (!commandsets) {
			GtkWidget *glade_menuitem = gtk_menu_item_new_with_label ("Default");
			gtk_widget_show (glade_menuitem);
			gtk_menu_append (GTK_MENU (optionmenu6_menu), glade_menuitem);
		} else {
			sLList *node = commandsets->objFirst;
			int n = 0;
			while (node) {
				if (node->obj && node->obj->getName()) {
					GtkWidget *glade_menuitem = gtk_menu_item_new_with_label ( node->obj->getName());
					(*mapCmdSet)[glade_menuitem] = node->obj->getName();
					gtk_widget_show (glade_menuitem);
					gtk_menu_append (GTK_MENU (optionmenu6_menu), glade_menuitem);
					if (bot && bot->objbot) {
						char *tmp = bot->objbot->getVar("commandset");
						if (tmp && !strcasecmp( tmp, node->obj->getName())) {
							gtk_menu_set_active ( (GtkMenu *)optionmenu6_menu, n);
						}
					}
					n++;
				}
				node = node->next;
			}
		}
		gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu6), optionmenu6_menu);
	}

  frame1 = gtk_frame_new ("Proxy");
  gtk_widget_ref (frame1);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "frame1", frame1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame1);
  gtk_box_pack_start (GTK_BOX (vbox1), frame1, TRUE, TRUE, 0);

  table3 = gtk_table_new (5, 2, FALSE);
  gtk_widget_ref (table3);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "table3", table3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table3);
  gtk_container_add (GTK_CONTAINER (frame1), table3);

  label10 = gtk_label_new ("Enable");
  gtk_widget_ref (label10);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "label10", label10,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label10);
  gtk_table_attach (GTK_TABLE (table3), label10, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (label10, 80, -2);
  gtk_misc_set_alignment (GTK_MISC (label10), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (label10), 2, 2);

  label11 = gtk_label_new ("Protocol");
  gtk_widget_ref (label11);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "label11", label11,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label11);
  gtk_table_attach (GTK_TABLE (table3), label11, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label11), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (label11), 2, 2);

  label12 = gtk_label_new ("Server");
  gtk_widget_ref (label12);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "label12", label12,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label12);
  gtk_table_attach (GTK_TABLE (table3), label12, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label12), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (label12), 2, 2);

  label13 = gtk_label_new ("Port");
  gtk_widget_ref (label13);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "label13", label13,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label13);
  gtk_table_attach (GTK_TABLE (table3), label13, 0, 1, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label13), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (label13), 2, 2);

  label14 = gtk_label_new ("UserName");
  gtk_widget_ref (label14);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "label14", label14,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label14);
  gtk_table_attach (GTK_TABLE (table3), label14, 0, 1, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label14), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (label14), 2, 2);

  entry7 = gtk_entry_new ();
  gtk_widget_ref (entry7);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "entry7", entry7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (entry7);
  gtk_table_attach (GTK_TABLE (table3), entry7, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 2, 1);

  entry8 = gtk_entry_new ();
  gtk_widget_ref (entry8);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "entry8", entry8,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (entry8);
  gtk_table_attach (GTK_TABLE (table3), entry8, 1, 2, 3, 4,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 2, 1);

  entry9 = gtk_entry_new ();
  gtk_widget_ref (entry9);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "entry9", entry9,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (entry9);
  gtk_table_attach (GTK_TABLE (table3), entry9, 1, 2, 4, 5,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 2, 1);
  gtk_entry_set_text (GTK_ENTRY (entry9), "Anonymous");

  optionmenu1 = gtk_option_menu_new ();
  gtk_widget_ref (optionmenu1);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "optionmenu1", optionmenu1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (optionmenu1);
  gtk_table_attach (GTK_TABLE (table3), optionmenu1, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 2, 1);
  optionmenu1_menu = gtk_menu_new ();
	{
		GtkWidget *glade_menuitem = gtk_menu_item_new_with_label ("No");
		gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu1_menu), glade_menuitem);
  glade_menuitem = gtk_menu_item_new_with_label ("Yes");
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu1_menu), glade_menuitem);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu1), optionmenu1_menu);
	}
	
  optionmenu2 = gtk_option_menu_new ();
  gtk_widget_ref (optionmenu2);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "optionmenu2", optionmenu2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (optionmenu2);
  gtk_table_attach (GTK_TABLE (table3), optionmenu2, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 2, 1);
  optionmenu2_menu = gtk_menu_new ();
	{
		GtkWidget *glade_menuitem = gtk_menu_item_new_with_label ("Socks4");
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu2_menu), glade_menuitem);
  glade_menuitem = gtk_menu_item_new_with_label ("Socks5");
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu2_menu), glade_menuitem);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu2), optionmenu2_menu);
	}

  hbuttonbox1 = gtk_hbutton_box_new ();
  gtk_widget_ref (hbuttonbox1);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "hbuttonbox1", hbuttonbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbuttonbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbuttonbox1, FALSE, TRUE, 0);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox1), GTK_BUTTONBOX_SPREAD);

  button1 = gtk_button_new_with_label ("Sweet!");
  gtk_widget_ref (button1);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "button1", button1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button1);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), button1);
  GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);

  button2 = gtk_button_new_with_label ("Cancel");
  gtk_widget_ref (button2);
  gtk_object_set_data_full (GTK_OBJECT (wndnewbot), "button2", button2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button2);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), button2);
  GTK_WIDGET_SET_FLAGS (button2, GTK_CAN_DEFAULT);

	if (bot && bot->objbot) {
		Obj *obj = bot->objbot;
		if (char *t = obj->getName())             gtk_entry_set_text( (GtkEntry *)entry1,  t);
		if (char *t = obj->getVar("passwd"))      gtk_entry_set_text( (GtkEntry *)entry2,  t);
		if (char *t = obj->getVar("server"))      gtk_entry_set_text( (GtkEntry *)entry3,  t);
		if (char *t = obj->getVar("register"))    gtk_entry_set_text( (GtkEntry *)entry4,  t);
		if (char *t = obj->getVar("cdkey"))       gtk_entry_set_text( (GtkEntry *)entry5,  t);
		if (char *t = obj->getVar("cdkey2"))      gtk_entry_set_text( (GtkEntry *)entry6,  t);
		if (char *t = obj->getVar("pserver"))     gtk_entry_set_text( (GtkEntry *)entry7,  t);
		if (char *t = obj->getVar("pport"))       gtk_entry_set_text( (GtkEntry *)entry8,  t);
		if (char *t = obj->getVar("puser"))       gtk_entry_set_text( (GtkEntry *)entry9,  t);
		if (char *t = obj->getVar("homechannel")) gtk_entry_set_text( (GtkEntry *)entry15, t);
		if (char *t = obj->getVar("active")) {
			if (atoi(t))
				gtk_option_menu_set_history ( GTK_OPTION_MENU (optionmenu3), 1);
			// gtk_menu_set_active( (GtkMenu *)optionmenu3_menu, 1);
		}
		if (char *t = obj->getVar("spawn")) {
			if (atoi(t))
				gtk_option_menu_set_history ( GTK_OPTION_MENU (optionmenu4), 1);
			//gtk_menu_set_active( (GtkMenu *)optionmenu4_menu, 1);
		}
		if (char *t = obj->getVar("client")) {
			for (int i = 0; i < WNDHASH_CLIENTS; i++)
				if (!strcasecmp( t, WndHash_clients[i]))
					gtk_option_menu_set_history ( GTK_OPTION_MENU (optionmenu5), i);
				// gtk_menu_set_active( (GtkMenu *)optionmenu5_menu, i);
		}

	}

  gtk_signal_connect (GTK_OBJECT (wndnewbot), "unrealize",
                      GTK_SIGNAL_FUNC (onWndNewBot_unrealize),
                      this);
  gtk_signal_connect (GTK_OBJECT (button1), "clicked",
                      GTK_SIGNAL_FUNC (onWndNewBot_button1_clicked),
                      this);
  gtk_signal_connect (GTK_OBJECT (button2), "clicked",
                      GTK_SIGNAL_FUNC (onWndNewBot_button2_clicked),
                      this);
	gtk_widget_show( wndnewbot);
}

WndNewBot::~WndNewBot( void) {
	delete mapCmdSet;
}
